#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   abaqus2fistr.py
#
#       abaqus から fistr へメッシュ変換
#       node番号、element番号の定義は、昇順に定義されている事が必要。
#       これらが昇順でない場合は、変換できるが、極端に実行速度が遅くなる。
#
#       abaqusType  node    fistrType
#       -------------------------------------------------------
#       C3D4        4       341             四面体1次（tetra）
#       C3D10       10      342             四面体2次
#       C3D8        8       361             六面体1次（hexa）
#       C3D8R       8       361              ↑
#       C3D20       20      362             六面体2次
#       C3D20R      20       ↑              ↑
#       C3D6        6       351             五面体1次（prism）
#       C3D15       15      352             五面体2次
#       STRI35      3       731,761(S341)   三角形1次表面要素
#       S3          3       731,761(S341)    ↑
#       S6          6       732(S342)       三角形2次表面要素
#       S4R5        4       741,781(S361)   四角形1次表面要素
#       S4          4       741,781(S361)    ↑
#       S8          8       (S362)          四角形2次表面要素
#       B31         2       611,641         beam
#       
#
#
#   15/03/19    新規作成
#      03/27    mashDataを数値で読み込みに変更
#      03/29    SGROUPを設定
#      03/30    5面体要素を追加
#      04/09    selValNameを修正
#      04/13    separateElset:高速化（直接global変数を操作する様に修正）
#      04/29    elementNoをインデックスでそのまま保存して検索を高速化
#      05/07    elementListデータをsortしてface面を検索（高速化の為）
#      05/08    nodeNoをインデクスでそのnodeを含むelement、typeのlist作成（高速化）
#      05/21    その他のsurGroupList「otherS」グループを作成
#      09/01    windows対応のため、os.sepに修正
#      09/02    "\n"→os.linesepに修正
#      10/18    getDataList:len(word)>0 and word!="\n"に修正
#               file読み込み時、"\n"行は読み飛ばす。
#      10/30    deleteLsOfLine:追加
#      12/22    changeNodeOrder351,changeNodeOrder352:バグ修正。
#               prismのnode順番が間違っていた為、四角形部にCLOADを設定して計算させた時
#               変位の正負が反対に計算されていた。これを修正。
#      12/24    getOtherSurGroupList:inpファイル内でsurfaceが定義されていない場合は、
#               エラーが発生していたので、修正。
#      12/25    行末コードlsを修正
#   16/01/08    getNodeElementData:inpファイルに「*NODE PRINT」が存在した場合エラー
#               発生したので、修正
#               saveData,checkNodeGroup:NSETのGENERATEに対応。
#      01/10    windows用のinpファイルを読み込んだ時、変換できない。
#               inpファイル読み込み時に、行末を"\n"に統一。
#               getDataList:座標が省略された場合にも対応できる様に修正
#      01/15    「*SURFACE」の定義の変換を追加
#               六面体1次2次要素の節点No順を修正（abaqusと同じ順に修正）
#      01/16    remakeElementHeaderData:EGRP名の設定を修正
#      02/03    getNodeElementData:大文字小文字の区別なくkeywordを取得
#      02/04    getDataList:deleteLsOfLine→deleteSpに変更
#      02/05    小文字のgroup名を残すように修正
#      03/24    createSurfaceNodesList:surfaceの定義が無い場合、ｴﾗｰ発生を修正
#      03/27    シェル要素（731、741）を追加
#      03/31    checkElementGroup:elTypeが複数ある場合にfindElementTypeを追加
#      04/01    複数のelTypeがあり、egrpを設定している場合、otherグループを設定
#      04/08    シェル要素（761、781）追加
#      04/11    getElementFaceNo:バグ修正（surData=[]の場所変更）
#      06/21    remakeSolidShellModel:追加（「!EQUATION」を追加する）
#      09/26    checkElementListOrder:node順の修正する位置を元に戻す
#               checkNoElementGroup:追加（EGRPが空白の場合、EGRPとしてeltypeを設定）
#      11/23    check641BeamElementGroup:NGRP名のdummyをdummyBeamに変更
#      11/27    remakeSolidShellModel:beamを追加
#      12/11    getNumberOfDummyNodesを追加（NGRPのdummyNGRPを作成）
#      12/18    checkDummyNodeGroup:dummyのNGRP中の最初と最後のNodeNoから要素を取得
#               （節点を共有する場合がある為、2点で要素を決定する）
#      12/23    checkDummyNodeGroup, getDummyNodes:高速化の為修正。
#               dummyNodes作成時にnodeToDummyListでnodeとdummy間のひも付け実施
#      12/25    checkDummyNodeGroup:バグ修正（dummyが必要ない要素の場合、ｴﾗｰ発生）
#   17/07/06    check641BeamElementGroup:beamを全て641に設定。
#      09/28    convNum2str:追加。座標などの実数を文字に変換する時、小数点有無を
#               チェックし、修正する。
#      10/28    getValName:大文字同士で検索する様に修正
#

import os
import sys

#ls = os.linesep           #行末コードを定義
ls = "\n"

abaName = ""
nodeList = []           #NODE
elementList = []        #ELEMENT
surfaceList = []        #ELEMENT（表面要素）
surGroupList = []       #SGROUP
surfaceGroupList = []   #SGROUP（*SURFACEで定義）
elsetList = []
ndsetList = []          #NGROUP
nElementType = 0        #elementTypeの種類
nElsetSur = []
nElsetEl = []
elNodeList = []         #element(表面、内部要素)ｘnodeNoのリスト（検索を早めるため）
nodeElsetList = []      #nodeNo?elementNoのリスト（検索を早めるため）
nodeToDummyList = []    #nodeNoに対するdummyNodeNoのリスト
dummyNodeList = []      #dummyNode取得の為のリスト

solidElements = [["341", 4], ["342", 10],   #四面体1次2次
                ["361", 8], ["362", 20],    #六面体1次2次
                ["351", 6], ["352", 15]]    #五面体1次2次

shellElements = [["731", 3], ["761", 6],    #三角形
                ["741", 4], ["781", 8]]     #四角形

beamElements = [["611", 2], ["641", 4]]     #梁要素


#  printMsg
#    print help
def printMsg():
    msg = """
<abaqus2fistr.py>
abaqusフォーマットのmeshをFrontISTR用meshに変換する。

<使い方>
abaqus2fistr.py <inpファイル>

例：
abaqus2fistr.py name.inp
abaqus2fistr.py name
"""
    print msg

#
#  getNodeElementData
#  ------------------
#    必要なdataを取得
def getNodeElementData(lines):
    item = ""
    header = ""
    data = []
    n = 0
    while n < len(lines):
        line = lines[n]
        if line[0] == "*":
            if len(data) > 0:
                saveData(item, header, data)

            #nodeの取得
            if line[:len("*NODE")].upper() == "*NODE":
                word = deleteSp(line).split(",")[0].upper()
                if word == "*NODE":
                    header = line
                    item = "NODE"
                    length = 4
                    data = []
            #要素の取得
            elif line[:len("*ELEMENT")].upper() == "*ELEMENT":
                typeName = getValName(line, "TYPE")
                #四面体1次
                if typeName == "C3D4":
                    header = setValName(line, "TYPE=341")
                    item = "ELEMENT"
                    length = 5
                    data = []
                #四面体2次
                elif typeName == "C3D10":
                    header = setValName(line, "TYPE=342")
                    item = "ELEMENT"
                    length = 11
                    data = []
                #六面体1次
                elif typeName == "C3D8" or typeName == "C3D8R":
                    header = setValName(line, "TYPE=361")
                    item = "ELEMENT"
                    length = 9
                    data = []
                #六面体2次
                elif typeName == "C3D20" or typeName == "C3D20R":
                    header = setValName(line, "TYPE=362")
                    item = "ELEMENT"
                    length = 21
                    data = []
                #五面体1次
                elif typeName == "C3D6":
                    header = setValName(line, "TYPE=351")
                    item = "ELEMENT"
                    length = 7
                    data = []
                #五面体2次
                elif typeName == "C3D15":
                    header = setValName(line, "TYPE=352")
                    item = "ELEMENT"
                    length = 16
                    data = []
                #表面要素（四面体1次）
                elif typeName == "STRI35" or typeName == "S3":
                    header = setValName(line, "TYPE=S341")
                    item = "SURFACE"
                    length = 4
                    data = []
                #表面要素（四面体2次）
                elif typeName == "S6":
                    header = setValName(line, "TYPE=S342")
                    item = "SURFACE"
                    length = 7
                    data = []
                #表面要素（六面体1次）
                elif typeName == "S4R5" or typeName == "S4":
                    header = setValName(line, "TYPE=S361")
                    item = "SURFACE"
                    length = 5
                    data = []
                #表面要素（六面体2次）
                elif typeName == "S8":
                    header = setValName(line, "TYPE=S362")
                    item = "SURFACE"
                    length = 9
                    data = []
                #beam1次
                elif typeName == "B31":
                    header = setValName(line, "TYPE=611")
                    item = "ELEMENT"
                    length = 3
                    data = []
                #以外の要素
                else:
                    item = ""
            #要素groupの取得
            elif line[:len("*ELSET")].upper() == "*ELSET":
                header = line
                item = "ELSET"
                length = 1
                data = []
            #nodeGroupの取得
            elif line[:len("*NSET")].upper() == "*NSET":
                header = line
                item = "NSET"
                length = 1
                data = []
            #surfaceGroupの取得
            elif line[:len("*SURFACE")].upper() == "*SURFACE":
                if deleteSp(line).split(",")[0] == "*SURFACE":
                    typeName = getValName(line, "TYPE")
                    if typeName == "NODE":
                        header = "*NSET\n"
                        item = "NSET"
                        length = 1
                        data = []
                    else:
                        header = line
                        item = "SURSET"
                        length = 2
                        data = []
            else:
                item = ""
            n += 1
        else:
            if item != "":
                [data, n] = getDataList(lines, n, item, length)
            else:
                n += 1
    if len(data) > 0:
        saveData(item, header, data)

#  getDataList
#    data部を取得
def getDataList(lines, n, item, length):
    data = []
    if item == "NODE":
        while n < len(lines) and lines[n][0] != "*":
            line = lines[n]
            words = line.split(",")
            a = [1, 0.0, 0.0, 0.0]
            for i in range(len(words)):
                if i == 0:
                    a[i] = int(words[0])
                else:
                    if len(words[i]) != 0:
                        a[i] = float(words[i])
            data.append(a)
            n += 1
    elif item == "ELEMENT" or item == "SURFACE":
        while n < len(lines) and lines[n][0] != "*":
            line = lines[n]
            #words = deleteLsOfLine(line).split(",")
            words = deleteSp(line).split(",")
            for word in words:
                if len(word) > 0 and word != "\n":
                    data.append(int(word))
            n += 1
        ans = []
        line = []
        p = 0
        for a in data:
            if p < length:
                line.append(a)
            else:
                ans.append(line)
                p = 0
                line = []
                line.append(a)
            p += 1
        ans.append(line)
        data = ans
    elif item == "ELSET" or item == "NSET":
        while n < len(lines) and lines[n][0] != "*":
            line = lines[n]
            words = deleteSp(line).split(",")
            for word in words:
                if len(word) > 0 and word != "\n":
                    try:
                        data.append(int(word))
                    except:
                        pass
            n += 1
    elif item == "SURSET":
        while n < len(lines) and lines[n][0] != "*":
            line = lines[n]
            words = deleteSp(line).split(",")
            if len(words) == 2:
                data.append([int(words[0]), words[1]])
            n += 1
    return [data, n]

#  saveData
#    読み込んだdataを取得する
def saveData(item, header, data):
    global nodeList, elementList, surfaceList, elsetList, ndsetList
    if item == "NODE":
        block = []
        num = str(len(data))
        header = "!NODE, NUM=" + num + ls
        block.append(header)
        block.append(data)
        nodeList.append(block)
        print "found NODE... num " + num
    elif item == "ELEMENT":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        header = ", ".join(words) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        elementList.append(block)
        name = getValName(header, "TYPE")
        print "found ELEMENT... TYPE " + name + " num " + num
    elif item == "SURFACE":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        header = ", ".join(words) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        surfaceList.append(block)
        name = getValName(header, "TYPE")
        print "found surfaceELEMENT... TYPE " + name + " num " + num
    elif item == "ELSET":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        newWords = []
        for word in words:
            if word == "GENERATE":
                data = generateNodeGroup(data)
                num = str(len(data))
            else:
                newWords.append(word)
        header = ", ".join(newWords) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        elsetList.append(block)
        name = getValName(header, "ELSET")
        print "found ELSET... " + name + " num " + num
    elif item == "NSET":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        newWords = []
        for word in words:
            if word == "GENERATE":
                data = generateNodeGroup(data)
                num = str(len(data))
            else:
                newWords.append(word) 
        header = ", ".join(newWords) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        ndsetList.append(block)
        name = getValName(header, "NSET")
        print "found NSET... " + name + " num " + num
    elif item == "SURSET":
        block = []
        num = str(len(data))
        name = getValName(header, "NAME")
        header = "!SGROUP, SGRP=" + name + ", NUM=" + num + ls
        block.append(header)
        data = convertFaceNo(data)
        block.append(data)
        surfaceGroupList.append(block)
        print "found surfaceGroup... NAME " + name + " num " + num

#
#  checkShellElementData
#  ---------------------
#    シェル要素のチェック。シェル要素がある場合、シェル要素を取得
def checkShellElementData():
    global elementList, surfaceList
    #solid要素が定義されているか
    #if len(elementList) != 0:
    if isThereSolidElement(elementList) == True:
        #solid要素が存在する場合、そのまま戻る
        return

    #シェル要素を取得（solid要素が存在しない場合）
    for n in range(len(surfaceList)):
        surface = surfaceList[n]
        surType = getValName(surface[0], "TYPE")
        if surType == "S341":
            #三角形1次
            surface[0] = setValName(surface[0], "TYPE=731")
            elementList.append(surface)
            surfaceList[n] = []
        elif surType == "S342":
            #三角形2次
            surface[0] = setValName(surface[0], "TYPE=732")
            elementList.append(surface)
            surfaceList[n] = []
        elif surType == "S361":
            #四角形1次
            surface[0] = setValName(surface[0], "TYPE=741")
            elementList.append(surface)
            surfaceList[n] = []

    #surfaceListを整形
    flag = 0
    newSurfaceList = []
    for surface in surfaceList:
        if len(surface) != 0:
            newSurfaceList.append(surface)
            flag = 1
    if flag == 0:
        surfaceList = []
    else:
        surfacelist = newSurfaceList

#  isThereSolidElement
#    solid要素が存在するかどうか確認する
def isThereSolidElement(elementList):
    ans = False
    for element in elementList:
        header = element[0]
        elType = getValName(header, "TYPE")
        for solType in solidElements:
            if elType == solType[0]:
                ans = True
                break
        if ans == True:
            break
    return ans

#
#  convertFaceNo
#    faceNoをfistr用に変換
def convertFaceNo(data):

    def getFaceNo(faceNo, nodes):
        face = int(faceNo[1:])
        nNode = len(nodes)
        if nNode == 4 or nNode == 10:
            #四面体1次2次要素
            faces = [1,2,3,4]
            cFace = faces[face-1]
        elif nNode == 8 or nNode == 20:
            #六面体1次2次要素
            #faces = [6,4,1,5,2,3]
            faces = [1,2,3,4,5,6]
            cFace = faces[face-1]
        elif nNode == 6 or nNode == 15:
            #五面体1次2次要素
            faces = [1,2,3,4,5]
            cFace = faces[face-1]
        return cFace

    for i in range(len(data)):
        elNo = data[i][0]
        faceNo = data[i][1]
        flag = 0
        for [header, elements] in elementList:
            for element in elements:
                if element[0] == elNo:
                    nodes = element[1:]
                    sNo = getFaceNo(faceNo, nodes)
                    data[i][1] = sNo
                    flag = 1
                    break
            if flag == 1:
                break
    return data

#
#  generateNodeGroup
#    GENERATEコマンドでnodeGroupを作成する
def generateNodeGroup(data):
    ist = int(data[0])
    ied = int(data[1])
    inc = 1
    if len(data) > 2:
        inc = data[2]
    newData = []
    i = ist
    while i <= ied:
        newData.append(i)
        i += inc
    return newData

#
#  checkElsetInElement
#    要素定義中のelsetをチェック
#    同じelsetが定義されていれば、これをまとめる
def checkElsetInElement():
    global elementList

    #  addToNewElementList
    #    同じelsetがある場合、newElementListに追加する
    def addToNewElementList(element, newElementList):
        flag = 0
        currElsetName = getValName(element[0], "ELSET")
        for newElement in newElementList:
            elsetName = getValName(newElement[0], "ELSET")
            if currElsetName == elsetName:
                #同じelsetに追加
                newElement[1] += element[1]
                flag = 1
                break
        if flag == 0:
            #新規に追加
            newElementList += [element]
        return newElementList

    newElementList = []
    for element in elementList:
        newElementList = addToNewElementList(element, newElementList)
    elementList = newElementList

#
#  checkElementListOrder
#  ---------------------
#    要素の並び順をチェック
def checkElementListOrder():
    global elementList, nElementType
    #elementの種類の個数を取得
    nElementType = len(elementList)
    #メッシュの並び順をチェック
    i = 0
    while i < len(elementList):
        element = elementList[i]
        header = element[0]
        typeName = getValName(header, "TYPE")
        #四面体2次
        if typeName == "342":
            data = changeNodeOrder342(element[1])
            elementList[i][1] = data
        #六面体1次
        elif typeName == "361":
            data = changeNodeOrder361(element[1])
            elementList[i][1] = data
        #六面体2次
        elif typeName == "362":
            data = changeNodeOrder362(element[1])
            elementList[i][1] = data
        #5面体1次
        elif typeName == "351":
            data = changeNodeOrder351(element[1])
            elementList[i][1] = data
        #5面体2次
        elif typeName == "352":
            data = changeNodeOrder352(element[1])
            elementList[i][1] = data
        #シェル三角形1次
        elif typeName == "731":
            data = changeNodeOrder731(element[1])
            elementList[i][1] = data
        #シェル三角形2次
        elif typeName == "732":
            data = changeNodeOrder732(element[1])
            elementList[i][1] = data
        #シェル四角形1次
        elif typeName == "741":
            data = changeNodeOrder741(element[1])
            elementList[i][1] = data
        #シェル三角形1次6節点
        elif typeName == "761":
            data = changeNodeOrder761(element[1])
            elementList[i][1] = data
        #シェル四角形1次8節点
        elif typeName == "781":
            data = changeNodeOrder781(element[1])
            elementList[i][1] = data
        i += 1
    #headerの修正
    remakeElementHeaderData()


#  changeNodeOrder342
#    四面体2次メッシュの節点順を変更
def changeNodeOrder342(data):
    i = 0
    while i < len(data):
        nums = data[i]
        newOder = [nums[0], nums[1], nums[2], nums[3], nums[4], nums[6], nums[7],
                nums[5], nums[8], nums[9], nums[10]]
        data[i] = newOder
        i += 1
    return data

#  changeNodeOrder361
#    六面体1次メッシュの節点順を変更
def changeNodeOrder361(data):
    i = 0
    while i < len(data):
        nums = data[i]
        #newOder = [nums[0], nums[1], nums[5], nums[6], nums[2], nums[4], nums[8],
        #        nums[7], nums[3]]
        newOder = [nums[0], nums[1], nums[2], nums[3], nums[4], nums[5], nums[6],
                nums[7], nums[8]]
        data[i] = newOder
        i += 1
    return data

#  changeNodeOrder362
#    六面体2次メッシュの節点順を変更
def changeNodeOrder362(data):
    i = 0
    while i < len(data):
        nums = data[i]
        #newOder = [nums[0], nums[1], nums[5], nums[6], nums[2], nums[4], nums[8],
        #        nums[7], nums[3], nums[17], nums[13], nums[18], nums[9], nums[20],
        #        nums[15], nums[19], nums[11], nums[12], nums[16], nums[14], nums[10]]
        newOder = [nums[0], nums[1], nums[2], nums[3], nums[4], nums[5], nums[6],
                nums[7], nums[8], nums[9], nums[10], nums[11], nums[12], nums[13],
                nums[14], nums[15], nums[16], nums[17], nums[18], nums[19], nums[20]]
        data[i] = newOder
        i += 1
    return data

#  changeNodeOrder351
#    五面体1次メッシュの節点順を変更
def changeNodeOrder351(data):
    i = 0
    while i < len(data):
        nums = data[i]
        #newOrder = [nums[0], nums[1], nums[3], nums[2], nums[4], nums[6], nums[5]]
        newOrder = [nums[0], nums[1], nums[2], nums[3], nums[4], nums[5], nums[6]]
        data[i] = newOrder
        i += 1
    return data

#  changeNodeOrder352
#    五面体2次メッシュの節点順を変更
def changeNodeOrder352(data):
    i = 0
    while i < len(data):
        nums = data[i]
        #newOrder = [nums[0], nums[1], nums[3], nums[2], nums[4], nums[6], nums[5],
        #        nums[8], nums[7], nums[9], nums[11], nums[10], nums[12], nums[13],
        #        nums[15], nums[14]]
        newOrder = [nums[0], nums[1], nums[2], nums[3], nums[4], nums[5], nums[6],
                nums[8], nums[9], nums[7], nums[11], nums[12], nums[10], nums[13],
                nums[14], nums[15]]
        data[i] = newOrder
        i += 1
    return data

#  changeNodeOrder731
#    シェル三角形1次メッシュの節点順を変更
def changeNodeOrder731(data):
    i = 0
    while i < len(data):
        nums = data[i]
        newOrder = [nums[0], nums[1], nums[2], nums[3]]
        data[i] = newOrder
        i += 1
    return data

#  changeNodeOrder732
#    シェル三角形2次要素の節点順を変更
def changeNodeOrder732(data):
    i = 0
    while i < len(data):
        nums = data[i]
        newOrder = [nums[0], nums[1], nums[2], nums[3], nums[5], nums[6], nums[4]]
        data[i] = newOrder
        i += 1
    return data

#  changeNodeOrder741
#    シェル四角形1次要素の節点順を変更
def changeNodeOrder741(data):
    i = 0
    while i < len(data):
        nums = data[i]
        newOrder = [nums[0], nums[1], nums[2], nums[3], nums[4]]
        data[i] = newOrder
        i += 1
    return data

#  changeNodeOrder761
#    761要素の節点順を変更
def changeNodeOrder761(data):
    return data

#  changeNodeOrder781
#    781要素の節点順を変更
def changeNodeOrder781(data):
    return data

#
#  remakeElementHeaderData
#    data整形
def remakeElementHeaderData():
    global elementList
    i = 0
    while i < len(elementList):
        header = elementList[i][0]
        typeName = getValName(header, "TYPE")
        num = getValName(header, "NUM")
        name = abaName.split(".")[0]
        egrpName = getValName(header, "ELSET")
        if (egrpName[:len("C3D")] == "C3D" or egrpName[:len("STR")] == "STR"
            or egrpName[:len("S6")] == "S6" or egrpName[:len("S4R")] == "S4R"):
            if nElementType == 1:
                egrpName = name
        header = "!ELEMENT, TYPE=" + typeName + ", PARTNAME=" + name + ", EGRP=" + egrpName + ", NUM=" + num + ls
        elementList[i][0] = header
        i += 1

#
#  checkSurfaceElementGroup
#  ------------------------
#    surfaceのgroup分けをチェック
def checkSurfaceElementGroup():
    print "checking data..."
    global surGroupList, elsetList, surfaceList, elementList

    #surfaceのlistを作成
    createSurfaceNodesList()

    #elsetをEGROUPとSGROUPに分ける
    surGroupList = []       #表面要素（surface）
    newElset = []           #内部要素（element）
    for elset in elsetList:
        surfaceNo = elset[1][0]
        if elNodeList[surfaceNo][0][0] == "S":
            #表面要素
            surGroupList += [elset]
        else:
            #内部要素
            newElset += [elset]
    elsetList = newElset
    #その他の表面要素を取得
    getOtherSurGroupList()
    #nodeを含むelementのlistを作成
    createNodeElementSet()

    print "creating SGROUP(surface)...",
    newSurElset = addNodesSurElset(surGroupList)        #各要素にnodeNoを追加
    [surfaceGroup, surElementSet] = getElementFaceNo(newSurElset)    #elementを取得
    surGroupList = remakeHeaderSurGroup(surfaceGroup)   #header部を修正
    if len(surfaceGroupList) > 0:
        for surfaceGroup in surfaceGroupList:           #「*SURFACE」内を追加
            if len(surfaceGroup[1]) > 0:
                surGroupList += [surfaceGroup]
                name = getValName(surfaceGroup[0], "SGRP")
                print name,
    if len(surElementSet) > 0:
        shellElementList = remakeHeaderShellElement(surElementSet, newSurElset)
        elementList += shellElementList
        if len(elsetList) > 0:
            shellGroups = getShellElementGroupList(shellElementList, newSurElset)
            elsetList += shellGroups
    print

#
#  getSolidElementGroupList
#    soild要素のelsetを取得
def getSolidElementGroupList(elementList):
    elset = []
    for element in elementList:
        line = element[0]
        egrp = getValName(line, "ELSET")
        header = "!ELSET, ELSET=" + egrp + ls
        data = []
        for elLine in element[1]:
            elNo = elLine[0]
            data.append(elNo)
        elset.append([header, data])
    return elset

#
#  getShellElementGroupList
#    shellのgroupを取得
def getShellElementGroupList(shellElementList, surElset):
    newElsetList = []
    for elset in surElset:
        setElNo = elset[1][0][0]
        for element in shellElementList:
            for elNode in element[1]:
                if setElNo == elNode[0]:
                    newElsetList.append(elset)
                    break
    ans = []
    if len(newElsetList) > 0:
        for elset in newElsetList:
            header = elset[0]
            data = []
            for element in elset[1]:
                data.append(element[0])
            ans.append([header, data])
    return ans

#
#  getOtherSurGroupList
#    その他の表面要素を取得
def getOtherSurGroupList():
    global surGroupList
    #配列を準備
    dummy = []
    for surface in surfaceList:
        for surNodes in surface[1]:
            dummy.append(surNodes[0])
    #surfaceを定義していない場合は、戻る
    if len(dummy) == 0:
        return

    maxSurNo = max(dummy)
    surElement = []
    i = 0
    while i < maxSurNo+1:
        surElement.append(-1)
        i += 1
    #elNoを取得
    for surface in surfaceList:
        for surNodes in surface[1]:
            elNo = surNodes[0]
            surElement[elNo] = 0
    #設定済の表面要素Noを取得
    for surGroup in surGroupList:
        for surElements in surGroup[1]:
            elNo = surElements
            surElement[elNo] = 1
    #その他の表面要素を取得
    otherData = []
    i = 0
    for surElNo in surElement:
        if surElNo == 0:
            otherData.append(i)
        i += 1
    #otherSのelsetを作成
    addList = []
    addList += ["!ELSET, ELSET=otherS, NUM=" + str(len(otherData)) + ls]
    addList += [otherData]
    surGroupList = [addList] + surGroupList

#
#  createNodeElementSet
#    nodeNoをindexにしてそのnodeを含むelementのlist作成
def createNodeElementSet():
    global nodeElsetList
    tmp = []
    for nodes in nodeList:
        for nodeLine in nodes[1]:
            tmp.append(nodeLine[0])
    if len(tmp) == 0:
        return

    maxNo = max(tmp)
    #配列の作成
    nodeElsetList = []
    i = 0
    while i < maxNo+1:
        nodeElsetList += [[[],[]]]
        i += 1
    #nodeLocを取得
    for nodes in nodeList:
        for nodeLine in nodes[1]:
            node = nodeLine[0]
            nodeElsetList[node][0] = nodeLine[1:]
    #elementNoを取得
    for elements in elementList:
        for nodesLine in elements[1]:
            elNo = nodesLine[0]
            nodes = nodesLine[1:]
            for node in nodes:
                nodeElsetList[node][1] += [elNo]

#
#  createSurfaceNodesList
#    表面要素の要素NoとnodeNoのlist作成
#    [[<"S" or "E">, elType], [node1, node2,...]]
def createSurfaceNodesList():
    global elNodeList
    #最大要素Noを取得
    tmp = []
    for surface in surfaceList:
        for surfaceNode in surface[1]:
            tmp.append(surfaceNode[0])
    for element in elementList:
        for elementNode in element[1]:
            tmp.append(elementNode[0])

    if len(tmp) == 0:
        return

    nMaxSurNo = max(tmp)
    #list作成
    elNodeList = []
    i = 0
    while i < nMaxSurNo+1:
        elNodeList.append([])
        i += 1
    #表面要素、要素typeを保存
    for surface in surfaceList:
        header = surface[0]
        elType = getValName(header, "TYPE")
        for surfaceNode in surface[1]:
            elNo = surfaceNode[0]
            elNodeList[elNo] = [["S", elType]] + surfaceNode[1:]
    #内部要素、要素typeを保存
    for element in elementList:
        header = element[0]
        elType = getValName(header, "TYPE")
        for elementNode in element[1]:
            elNo = elementNode[0]
            elNodeList[elNo] = [["E", elType]] + elementNode[1:]

#  separateElset
#    elsetをEGROUPとSGROUPに分ける
def separateElset():
    global surfaceList, surGroupList, elsetList
    surGroupList = []
    newElset = []
    i = 0
    while i < len(elsetList):
        elset = elsetList[i]
        surfaceNo = elset[1][0]
        flag = 0
        for surElement in surfaceList:
            for surElNo in surElement[1]:
                if surfaceNo == surElNo[0]:
                    flag = 1
                    break
            if flag == 1:
                break
        if flag == 1:
            surGroupList += [elset]
        else:
            newElset += [elset]
        i += 1
    elsetList = newElset
    return

#  addNodesSurElset
#    各表面要素にnode番号を追加
def addNodesSurElset(elsets):
    newSurElset = []
    for elset in elsets:
        data = []
        for elNo in elset[1]:
            nodes = elNodeList[elNo][1:]
            newNodes = [elNo] + nodes
            data.append(newNodes)
        data = [elset[0]] + [data]
        newSurElset.append(data)
    return newSurElset

#  findSurfaceNodes
def findSurfaceNodes(elNo):
    ans = []
    flag = 0
    for surface in surfaceList:
        for nodeNo in surface[1]:
            if nodeNo[0] == elNo:
                flag = 1
                ans = nodeNo[1:]
                break
        if flag == 1:
            break
    return ans

#  getElementFaceNo
#    elementとfaceNo取得、shell要素を取得
def getElementFaceNo(elsets):
    #elementNo取得
    elementNoSet = []       #solidElements
    surElementSet = []      #shellElements
    surData = []
    for elset in elsets:
        header = elset[0]
        name = getValName(header, "ELSET")
        data = []
        #surData = []
        ellNo = 0
        for line in elset[1]:
            setNodes = line[1:]
            [elementNo, faceNo] = findElementNo(setNodes)
            if elementNo != "":
                data += [[elementNo] + [faceNo]]
            else:
                surData += [line]
        if len(data) > 0:
            print name,
            data.sort()
            elementNoSet += [[header] + [data]]
    if len(surData) > 0:
        surData.sort()
        surElementSet = makeShellElementSet(surData)
    else:
        surElementSet = []
    return [elementNoSet, surElementSet]

#
#  makeShellElementSet
#    shell要素のsetを取得
def makeShellElementSet(surData):
    elTypes = [["", []]]
    for surface in surData:
        elNo = surface[0]
        elType = elNodeList[elNo][0][1]
        flag = 0
        for i in range(len(elTypes)):
            if elTypes[i][0] == elType:
                elTypes[i][1].append(surface)
                flag = 1
                break
        if flag == 0:
            elTypes.append([elType, [surface]])
    return elTypes[1:]

#  findElementNo
#    elementNoを取得し、そのfaceNoを取得して返す
def findElementNo(nodes):
    flag = 0
    i = 0
    #最初のnodeNoを含むelementListを取得
    elements = nodeElsetList[nodes[0]][1]
    while i < len(elements):
        if i < len(elements):
            elementNo = elements[i]
            line = elNodeList[elementNo]
            if isInNodes(line[1:], nodes) == True:
                elType = elNodeList[elementNo][0][1]
                faceNo = getFaceNoOfElement(line[1:], nodes, elType)
                flag = 1
                break
        i += 1
    if flag == 1:
        ans = [elementNo, faceNo]
    else:
        ans = ["", ""]
    return ans

#  isInNodes
def isInNodes(line, nodes):
    flag = 0
    ans = False
    for node in nodes:
        if node in line:
            flag += 1
        else:
            break
    if flag == len(nodes):
        ans = True
    return ans

#  remakeHeaderSurGroup
#    headerを書き換え
def remakeHeaderSurGroup(elsets):
    i = 0
    while i < len(elsets):
        elset = elsets[i]
        header = elset[0]
        groupName = getValName(header, "ELSET")
        num = getValName(header, "NUM")
        partName = abaName.split(".")[0]
        header = "!SGROUP, PARTNAME=" + name + ", SGRP=" + groupName + ", NUM=" + num + ls
        elsets[i][0] = header
        i += 1
    return elsets

#  remakeHeaderShellElement
#    headerを書き換え
def remakeHeaderShellElement(surElementSet, newElset):
    global elNodeList, nodeList, dummyNodeList, nodeToDummyList
    #配列を準備
    dummyNodeList = []
    maxNodeNo = len(nodeElsetList)
    for node in range(maxNodeNo+1):
        dummyNodeList.append("")
        nodeToDummyList.append("")
    currNo = maxNodeNo
    newNodes = []
    for i in range(len(surElementSet)):
        surType = surElementSet[i][0]
        if surType == "S341":
            elType = "761"
            elset = "S3"
        elif surType == "S361":
            elType = "781"
            elset = "S4"
        #surElementListのデータを修正
        header = "!ELEMENT, TYPE=" + elType + ", ELSET=" + elset + ls
        surElementSet[i][0] = header
        #元々のデータ（elNodeList）も修正
        for j in range(len(surElementSet[i][1])):
            elNo = surElementSet[i][1][j][0]
            [addNodes, currNo] = createNewNodes(surElementSet[i][1][j], currNo)
            elNodeList[elNo][0] = ["E", elType]
            elNodeList[elNo] += addNodes
            surElementSet[i][1][j] += addNodes
    newNodes = getDummyNodes()
    header = "!NODE, NGRP=dummy" + ls
    nodeList.append([header, newNodes])
    return surElementSet

#  getDummyNodes
#
def getDummyNodes():
    global nodeToDummyList
    nodeData = []
    for i in range(len(dummyNodeList)):
        nodeNo = dummyNodeList[i]
        if nodeNo != "":
            nodeLoc = nodeElsetList[i][0]
            nodeData.append([nodeNo] + nodeLoc)
            nodeToDummyList[i] = nodeNo
    nodeData.sort()
    return nodeData

#  createNewNodes
#    同じ座標のnodeを新しく作成する
def createNewNodes(nodes, currNo):
    global nodeElsetList, dummyNodeList
    #newNo = len(nodeElsetList)
    nums = []
    for node in nodes[1:]:
        if dummyNodeList[node] == "":
            nodeLoc = nodeElsetList[node][0]
            nodeElsetList.append([nodeLoc,[nodes[0]]])
            nums.append(currNo)
            dummyNodeList[node] = currNo
            currNo += 1
        else:
            dummy = dummyNodeList[node]
            nums.append(dummy)
    return [nums, currNo]

#  getFaceNoOfElement
#    elementのfaceNo取得
def getFaceNoOfElement(elNodes, faceNodes, elType):
    faceNo = 0
    #四面体（1次、2次）要素の場合
    if elType == "341" or elType == "342":
        nodes = [1,2,3,4]
        i = 0
        newNodes = []
        while i < len(nodes):
            if elNodes[i] in faceNodes:
                newNodes.append(i+1)
            i += 1
        newNodes.sort()
        if newNodes == [1,2,3]:
            faceNo = 1
        elif newNodes == [1,2,4]:
            faceNo = 2
        elif newNodes == [2,3,4]:
            faceNo = 3
        elif newNodes == [1,3,4]:
            faceNo = 4
        else:
            faceNo = 0
    #六面体（1次、2次）要素の場合
    elif elType == "361" or elType == "362":
        nodes = [1,2,3,4,5,6,7,8]
        i = 0
        newNodes = []
        while i < len(nodes):
            for faceNode in faceNodes:
                if elNodes[i] == faceNode:
                    newNodes.append(nodes[i])
                    break
            i += 1
        newNodes.sort()
        if newNodes == [1,2,3,4]:
            faceNo = 1
        elif newNodes == [5,6,7,8]:
            faceNo = 2
        elif newNodes == [1,2,5,6]:
            faceNo = 3
        elif newNodes == [2,3,6,7]:
            faceNo = 4
        elif newNodes == [3,4,7,8]:
            faceNo = 5
        elif newNodes == [1,4,5,8]:
            faceNo = 6
        else:
            faceNo = 0
    #五面体（1次、2次）要素の場合
    elif elType == "351" or elType == "352":
        nodes = [1,2,3,4,5,6]
        i = 0
        newNodes = []
        while i < len(nodes):
            for faceNode in faceNodes:
                if elNodes[i] == faceNode:
                    newNodes.append(nodes[i])
                    break
            i += 1
        newNodes.sort()
        if newNodes == [1,2,3]:
            faceNo = 1
        elif newNodes == [4,5,6]:
            faceNo = 2
        elif newNodes == [1,2,4,5]:
            faceNo = 3
        elif newNodes == [2,3,5,6]:
            faceNo = 4
        elif newNodes == [1,3,4,6]:
            faceNo = 5
        else:
            faceNo = 0
    return faceNo

#
#  checkBeamElementGroup
#  ---------------------
#    beam要素（611）の取得
def checkBeamElementGroup():
    global elementList
    #全てのbeam要素を取得
    beamList = []
    for element in elementList:
        header = element[0]
        elType = getValName(header, "TYPE")
        if elType == "611" or elType == "641":
            beamList.append(element)
    #表面要素に含まれるbeam要素かどうかチェック
    elNoList = []
    for beam in beamList:
        for i in range(len(beam[1])):
            beam[1][i] += ["N"]
    newBeamList = []
    #表面要素を取得
    surfaceElement = []
    for surface in surfaceList:
        if len(surface) >= 0:
            surfaceElement += surfaceList
    for element in elementList:
        header = element[0]
        elType = getValName(header, "TYPE")
        for elName in shellElements:
            if elName[0] == elType:
                surfaceElement.append(element)
                break
    #表面要素に含まれないbeam要素を取得
    if len(surfaceElement) == 0:
        #表面要素が存在しない場合
        for beam in beamList:
            header = beam[0]
            data = []
            for beamLine in beam[1]:
                data.append(beamLine[:-1])
            newBeamList.append([header, data])
    else:
        #表面要素に含まれないbeam要素を取得
        for surfaces in surfaceElement:
            for surface in surfaces[1]:
                surElNo = surface[0]
                surNodes = surface[1:]
                for i in range(len(beamList)):
                    beam = beamList[i]
                    if beam[-1] == "N":
                        beamNodes = beam[1:]
                        if isInNodes(surNodes, beamNodes) == True:
                            newBeamList.append([surElNo, nodes[:-1]])
                            elNoList[i][-1] = "Y"
    #elementListからbeam要素を削除
    newElementList = []
    for element in elementList:
        header = element[0]
        elType = getValName(header, "TYPE")
        if not (elType == "611" or elType == "641"):
            newElementList.append(element)
    elementList = newElementList
    if len(newBeamList) != 0:
        #beam要素を追加
        elementList = newBeamList + elementList
    return

#
#  check641BeamElementGroup
#  ------------------------
#    641要素（solidとbeam混在型）を取得
def check641BeamElementGroup():
    global elementList, nodeList, elNodeList, dummyNodeList, nodeToDummyList
    """
    flag = 0
    for element in elementList:
        elType = getValName(element[0], "TYPE")
        if elType == "341" or elType== "361" or elType == "351":
            flag = 1
            break
    if flag == 0:
        return
    """

    #beam要素611を641に修正（solidが存在する場合）
    flag = 0
    i = 0
    for element in elementList:
        elType = getValName(element[0], "TYPE")
        if elType == "611":
            flag = 1
            header = element[0]
            header = setValName(header, "TYPE=641")
            elementList[i][0] = header
        i += 1
    #beamが無い場合は、すぐに戻る
    if flag == 0:
        return

    #dummyNodeを追加
    #  配列を準備
    dummyNodeList = []
    maxNodeNo = len(nodeElsetList)
    for node in range(maxNodeNo+1):
        dummyNodeList.append("")
        nodeToDummyList.append("")
    currNo = maxNodeNo
    newNodes = []
    for i in range(len(elementList)):
        header = elementList[i][0]
        elType = getValName(header, "TYPE")
        if elType == "641":
            for j in range(len(elementList[i][1])):
                elNo = elementList[i][1][j][0]
                orgNodes = elementList[i][1][j][1:]
                [addNodes, currNo] = createNewNodes(elementList[i][1][j], currNo)
                elNodeList[elNo] += addNodes
                elementList[i][1][j] += addNodes
    newNodes = getDummyNodes()
    header = "!NODE, NGRP=dummyBeam" + ls
    nodeList.append([header, newNodes])
    return

#
#  checkElementGroup
#  -----------------
#    elementのgroup分けをチェック
def checkElementGroup():
    global elsetList, elementList, nElementType
    if len(elsetList) == 0:
        return

    print "creating EGROUP(element)...",
    #elementTypeの数は？
    if nElementType > 1:
        #複数のelementTypeがある場合
        if len(elsetList) > 0:
            #要素grpを設定している場合
            newElementList = []
            newElGroup = []
            for elset in elsetList:
                newData = createNewElGroup(elset)
                newElGroup.append(newData)
            elsetList = newElGroup
            for n in range(len(elsetList)):
                elset = elsetList[n]
                newData = createNewElementData(elset)
                elType = findElementType(newData[1])
                newData[0] = setValName(newData[0], "TYPE="+elType)
                newElementList.append(newData)
            otherElementList = createOtherElementForElType(newElementList)
            elementList = newElementList + otherElementList
            elsetList = []
        print
        return

    #elementTypeが1種類の場合
    newElementList = []
    flag = 0
    for elset in elsetList:
        flag = 1
        newData = createNewElementData(elset)
        newElementList.append(newData)
    if flag == 1:
        otherData = createOtherElementData(newElementList)
        if len(otherData[1]) > 0:
            newElementList.append(otherData)
        elementList = newElementList
    print

#  findElementType
#    要素Noからeltypeを検索する
def findElementType(elData):
    elNo = elData[0][0]
    element = elNodeList[elNo]
    elType = element[0][1]
    return elType

#  createOtherElementForElType
#    elType毎のotherElemntを取得
def createOtherElementForElType(newElementList):
    #配列を準備
    elset = []
    i = 0
    for i in range(len(elNodeList)):
        elset.append(0)
    #定義済みの場合、配列に「1」をセット
    for element in newElementList:
        for elNode in element[1]:
            elNo = elNode[0]
            elset[elNo] = 1
    #未定義のelementNoを取得
    otherData = []
    for i in range(len(elset)):
        if elset[i] == 0:
            otherData.append(i)
    #otherElementを取得
    otherElement = []
    for i in range(len(elementList)):
        header = elementList[i][0]
        elType = getValName(header, "TYPE")
        otherElement.append([elType, []])
    nType = len(otherElement)
    for elNo in otherData:
        element = elNodeList[elNo]
        if len(element) > 0:
            for i in range(nType):
                if otherElement[i][0] == element[0][1]:
                    otherElement[i][1].append([elNo] + element[1:])
                    break
    #headerを整形
    otherElementList = []
    for element in otherElement:
        if len(element[1]) > 0:
            otherElementList.append(element)
    for i in range(len(otherElementList)):
        elType = otherElementList[i][0]
        egrp = "other" + elType
        header = "!ELEMENT, TYPE=" + elType + ", EGRP=" + egrp + ls        
        otherElementList[i][0] = header
        print egrp,
    return otherElementList

#  createNewElGroup
#    elsetを作成する
def createNewElGroup(elset):
    line = elset[0]
    name = getValName(line, "ELSET")
    partName = abaName.split(".")[0]
    num = getValName(line, "NUM")
    header = "!EGROUP, PARTNAME=" + partName + ", EGRP=" + name + ", NUM=" + num + ls
    elset[0] = header
    return elset

#  createNewElementData
#    elsetのelementDataを作成する
def createNewElementData(elset):
    line = elset[0]
    name = getValName(line, "ELSET")
    if name == "":
        name = getValName(line, "EGRP")
    line = elementList[0][0]
    header = setValName(line, "EGRP="+name)
    print name,
    ans = []
    newElset = []
    for elNo in elset[1]:
        nodes = elNodeList[elNo][1:]
        newElset += [[elNo] + nodes]
    num = str(len(newElset))
    header = setValName(header, "NUM="+num)
    ans.append(header)
    ans.append(newElset)
    return ans

#  createOtherElementData
#    otherElementを追加する
def createOtherElementData(newElementList):
    nEle = 0
    for element in newElementList:
        nEle += len(element[1])
    if nEle == len(elementList[0][1]):
        ans = ["", []]
        return ans

    #既に設定してあるELSETを取得
    #  配列を準備
    elset = []
    i = 0
    while i < len(elNodeList):
        elset.append(0)
        i += 1
    #  定義済の場合、「1」をセット
    for element in newElementList:
        for elNode in element[1]:
            elNo = elNode[0]
            elset[elNo] = 1

    #otherを取得
    print "other",
    otherData = []
    for elLine in elementList[0][1]:
        flag = 0
        elNo = elLine[0]
        if elset[elNo] == 0:
            otherData.append(elLine)
    header = elementList[0][0]
    header= setValName(header, "EGRP=other")
    num = str(len(otherData))
    header = setValName(header, "NUM="+num)
    ans = [header]
    ans.append(otherData)
    return ans

#
#  checkNodeGroup
#  --------------
#    nodeGroupをチェック
def checkNodeGroup():
    global ndsetList
    print "creating NGROUP(node)...",
    newNdset = []
    for ndSet in ndsetList:
        data = []
        header = ndSet[0]
        name = getValName(header, "NSET")
        print name,
        num = getValName(header, "NUM")
        partName = abaName.split(".")[0]
        newHeader = "!NGROUP, PARTNAME=" + partName + ", NGRP=" + name
        if header.find("GENERATE") >= 0:
            newHeader += ", GENERATE"
        newHeader += ", NUM=" + num + ls
        data.append(newHeader)
        data.append(ndSet[1])
        data[1].sort()
        newNdset.append(data)
    ndsetList = newNdset
    print

#
#  checkDummyNodeGroup
#  -------------------
#    NGRPに対応したdummyのnodeGroupを作成する。
def checkDummyNodeGroup():
    global ndsetList
    print "creating dummyNGROUP(node)...",
    #dummyNodeを取得
    newNdset = []
    for ndset in ndsetList:
        header = ndset[0]
        data = ndset[1]
        newData = []
        for nodeNo in data:
            if nodeNo < len(nodeToDummyList):
                dummyNode = nodeToDummyList[nodeNo]
                if dummyNode != "":
                    newData.append(dummyNode)
                #node数が同じ場合、取得
                if len(newData) == len(data):
                    ngrpName = "dummy_" + getValName(header, "NGRP")
                    print ngrpName,
                    header = setValName(header, "NGRP="+ngrpName)
                    newNdset.append([header, newData])
    print
    ndsetList += newNdset

#
#  checkNoElementGroup
#  -----------------
#    メッシュに要素groupを設定しなかった場合の対処
def checkNoElementGroup():
    global elementList
    for element in elementList:
        header = element[0]
        name = getValName(header, "EGRP")
        if name == "":
            #EGRPが空白の場合は、EGRPとしてelTypeを設定する
            elType = getValName(header, "TYPE")
            newHeader = "!ELEMENT, TYPE=" + elType + "\n"            
            newHeader = setValName(newHeader, "EGRP=E"+elType)
            element[0] = newHeader

#
#  witeData
#  --------
#    dataの書き込み
def writeData(fileName):
    print "writing...",
    cont = makeHeader()
    cont += makeNode()
    cont += makeElement()
    cont += makeElementGroup()
    cont += makeSurfaceGroup()
    cont += makeNodeGroup()
    cont += "!END"
    f=open(fileName, "w"); f.write(cont); f.close()
    print "done"

#  makeHeader
#    header
def makeHeader():
    cont = "!HEADER" + ls
    cont += "  genarated by abaqus2fistr.py" + ls
    return cont

#  makeNode
#    node
def makeNode():
    cont = ""
    for node in nodeList:
        cont += node[0]
        for nums in node[1]:
            strs = []
            for num in nums:
                word = convNum2str(num)
                #指数表記で「.」が無い場合、「.」を追加する。
                if word.find(".") < 0:
                    n = word.find("e")
                    if n >= 0:
                        word = word[:n] + ".0" + word[n:]
                strs.append(word)
            cont += ", ".join(strs) + ls
    return cont        

#  makeElement
#    element
def makeElement():
    cont = ""
    for element in elementList:
        header = element[0]
        name = getValName(header, "EGRP")
        elType = getValName(header, "TYPE")
        cont += header
        for nums in element[1]:
            strs = []
            for num in nums:
                strs.append(str(num))
            cont += ", ".join(strs) + ls
        if elType[0] == "7":
            section = "!SECTION, TYPE=SHELL, EGRP=" + name + ls
            section += "1.0, 5" + ls
        elif elType[0] == "6":
            section = "!SECTION, TYPE=BEAM, EGRP=" + name + ls
            section += "0.0, 0.0, -1.0, 1.0, 1.0, 1.0, 1.0" + ls
        else:
            section = "!SECTION, TYPE=SOLID, EGRP=" + name + ls
        cont += section
    return cont

#  makeElementGroup
#    elementGroup
def makeElementGroup():
    cont = ""
    if nElementType == 1:
        return cont

    for elGroup in elsetList:
        cont += elGroup[0]
        cont += makeStringFromData(elGroup[1], 8)
    return cont

#  makeSurfaceGroup
#    表面要素group
def makeSurfaceGroup():
    cont = ""
    for surGroup in surGroupList:
        cont += surGroup[0]
        for surSet in surGroup[1]:
            strs = []
            for num in surSet:
                strs.append(str(num))
            cont += ", ".join(strs) + ls
    return cont

#  makeNodeGroup
#    nodeGroup
def makeNodeGroup():
    cont = ""
    if len(ndsetList) == 0:
        return cont

    for ndset in ndsetList:
        cont += ndset[0]
        cont += makeStringFromData(ndset[1], 8)
    return cont

#  makeStringFromData
#    dataSetをstrに変換
def makeStringFromData(data, w):
    cont = ""
    p = 0
    while p+w < len(data):
        nums = data[p:p+w]
        strs = []
        for num in nums:
            strs.append(str(num))
        cont += ", ".join(strs) + "," + ls
        p += w
    nums = data[p:]
    strs = []
    for num in nums:
        strs.append(str(num))
    cont += ", ".join(strs) + ls
    return cont

#  getValName
#    valName(TYPEなど）を取得
def getValName(line, valName):
    valName = valName.upper()
    words = line.split(",")
    name = ""
    for word in words:
        # 「TYPE=」などを検索
        if word.upper().find(valName+"=") >= 0:
            name = word.split("=")[1]
            break
    newName = ""
    for chara in name:
        if chara > " ":
            newName += chara
    return newName

#  deleteSp
#    空白を削除
def deleteSp(line):
    newLine = ""
    for chara in line:
        if chara != " " and chara != "\n" and chara != "\r":
            newLine += chara
    return newLine

#  setValName
#    変数の値をセット
#    行末コードにより処理を変更（importして使う事がある為）
#    　abaqus2fistrの中のみなら問題なし
def setValName(line, val):
    valNames = val.split("=")
    #words = (line.split(ls)[0]).split(",")
    words = deleteLsOfLine(line).split(",")
    i = 0
    flag = 0
    for word in words:
        name = (word.split()[0]).split("=")[0].upper()
        if name == valNames[0]:
            words[i] = " " + val
            flag = 1
            break
        i += 1
    if flag == 0:
        words.append(" "+val)
    newLine = ",".join(words) + ls
    return newLine

#  deleteLsOfLine
#    行末コードを削除する
def deleteLsOfLine(line):
    if line[-2:] == "\r\n":
        newLine = line[:-2]
    else:
        newLine = line[:-1]
    return newLine

#
#  convNum2str
#    数値を文字列に変換する
#    指数表記で小数点が無い場合は、少数点を追加する
def convNum2str(num):
    word = str(num)
    if word.find(".") >= 0:
        return word
    n = word.find("e")
    if n >= 0:
        word = word[:n] + ".0" + word[n:]
    return word

#
#  remakeSolidShellModel
#    solidSellの混在モデルの場合、EQUATIONを追加
def remakeSolidShellModel(fileName):
    comm = "remakeSolidShellBeamModel.py " + fileName
    os.system(comm)

#
#  abaqus2fistr
#  ------------
#    convert abaqus to fistr
def abaqus2fistr(fileDir, abaName):
    print
    print "abaqus mesh will be convert to FrontISTR."
    fileName = fileDir + os.sep + abaName + ".inp"
    f = open(fileName)
    lines = []
    for line in f.readlines():
        if line != "\n" and line !="\r\n" :
            if line[-2:] == "\r\n":
                line = line[:-2] + "\n"
            lines.append(line)
    f.close()
    #データ取得
    getNodeElementData(lines)
    #シェル要素をチェックし、シェル要素（731、741）があれば取得
    checkShellElementData()
    #要素定義中のelsetをチェック（同じ要素名の場合、まとめる）
    checkElsetInElement()
    #nodeの並びをチェック
    checkElementListOrder()
    #表面groupの取得,shell要素（761、781）取得
    checkSurfaceElementGroup()
    #beam要素（611）取得
    checkBeamElementGroup()
    #EGRPのチェック修正（要素groupを設定しなかった場合）
    checkNoElementGroup()
    #要素groupの取得（単一要素の場合は、要素定義をgroup分けする）
    checkElementGroup()

    #beam要素（641）を取得
    check641BeamElementGroup()

    #nodeGroupの取得
    checkNodeGroup()
    checkDummyNodeGroup()
    #書き出し(header等をremake）
    newName = fileDir + os.sep + abaName + ".msh"
    writeData(newName)
    #EQUATIONを追加（solidShellの場合）
    remakeSolidShellModel(newName)


if __name__=='__main__':
    abaNameDir = sys.argv[1]
    if abaNameDir[:len("-h")] == "-h" or abaNameDir[:len("--help")] == "--help":
        printMsg()
    else:
        fileDir = os.sep.join(abaNameDir.split(os.sep)[:-1])
        if fileDir == "":
            fileDir = os.getcwd()
        abaName = abaNameDir.split(os.sep)[-1]
        if abaName.split(".")[-1] == "inp":
            name = abaName[:-4]
        else:
            name = abaName
        abaqus2fistr(fileDir, name)

