#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   fistr2vtk.py
#
#       fistrからvtkにファイル変換する
#
#       fistr   node    vtk
#       -----------------------------------------
#       341     4       10      四面体1次
#       342     10      24      四面体2次
#       361     8       12      六面体1次
#       362     20      25      六面体2次
#       351     6       13      五面体1次
#       352     15      26      五面体2次
#       611     2       3       beam要素
#       641     4       3       beam要素（3自由度）
#       731     3       5       シェル三角形1次
#       732     6       22      シェル三角形2次
#       741     4       9       シェル四角形1次
#       761     6       5       シェル三角形1次（3自由度）
#       781     8       9       シェル四角形1次（3自由度）
#
#
#   15/05/19    新規作成
#      05/21    faceGroupの値を「1」→「0」からに変更
#               surfaceVtkフォルダ内に表面要素のvtkを作成（表面よvolume要素は別ファイル）
#      05/23    表面要素とvolume要素を１ヶのfileに設定
#               elementGroup 0     :表面要素
#                            1以上 :volume要素のelementGroupNo
#               faceGroup    0     :volume要素
#                            1以上 :表面要素のfaceGroupNo
#      06/08    結果fileにpointデータのみの場合があるので、これに対応
#      09/02    "\n"→os.linesepに修正
#      10/18    print u""形式に全て修正。
#               ファイル読み込み時、"\n"行は、読み飛ばす
#      12/25    行末コードlsを修正
#   16/03/27    シェル要素（731、741）追加
#               createVtkFileResult:DISPLACEMENT内に回転が含まれている場合は、削除
#      04/08    シェル要素（761、781）追加
#      06/11    梁要素（641）追加
#      06/27    checkNanData:nodeDataが「NAN」の場合、「0.0」に修正する事を追加
#      11/08    getNodesAllArrayData:!NODE内のコメント行をskipするように修正
#   17/09/27    高速化の為、ファイルの書き出し方法を修正
#      10/08    getNodeNo:バグ修正（未使用のnodeNoも取得していた。）
#
#

import os
import sys
import glob

#                       vtkのcell定義行          vtkのcell fistrのNo
cellConts = []      #[[[node数, node0, node1...], cell名, elementNo],...]
nodeConts = []
nodeGrpNo = []
faceGrpNo = []
#ls = os.linesep     #行末コード
ls = "\n"

#  cellTypeの定義、変換
def cellTypesCont():
    #        [<要素Type>, <node数>, <vtk要素type>]
    names = [["341", 4, "10"], ["342", 10, "24"],
             ["361", 8, "12"], ["362", 20, "25"],
             ["351", 6, "13"], ["352", 15, "26"],
             ["731", 3, "5"],  ["732", 6,  "22"],
             ["741", 4, "9"],  ["781", 8, "12"],
             ["761", 6, "13"],  ["611", 2, "3"],
             ["641", 4, "3"]]
    return names

#  fisr要素をvtk要素に変換する時のnodeNoの並び順
#    342, 352の要素並びが異なっている
def nodeOrder():
    order = [["341", [0,1,2,3]],
             ["342", [0,1,2,3,6,4,5,7,8,9]],
             ["361", [0,1,2,3,4,5,6,7]],
             ["362", [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19]],
             ["351", [0,1,2,3,4,5]],
             ["352", [0,1,2,3,4,5,8,6,7,11,9,10,12,13,14]],
             ["731", [0,1,2]],
             ["732", [0,1,2,5,3,4]],
             ["741", [0,1,2,3]],
             ["781", [0,1,2,3,4,5,6,7]],
             ["761", [0,1,2,3,4,5]],
             ["611", [0,1]],
             ["641", [0,1,2,3]]]
    return order

#  vtk要素のfaceを構成するnodeNo
#   nodeNoの起点は「0」
def faceNodeOfElement():
    data = [["10", [[0,1,2], [0,1,3], [1,2,3], [2,0,3]]],
            ["24", [[0,4,1,5,2,6], [0,4,1,8,3,7], [1,5,2,9,3,8], [2,6,0,7,3,9]]],
            ["12", [[0,1,2,3], [4,5,6,7], [0,1,5,4], [1,2,6,5], [2,3,7,6],
                     [3,0,4,7]]],
            ["25", [[0,8,1,9,2,10,3,11], [4,12,5,13,6,14,7,15],
                    [0,8,1,17,5,12,4,16], [1,9,2,18,6,13,5,17],
                    [2,10,3,19,7,14,6,18], [3,11,0,16,4,15,7,19]]],
            ["13", [[0,1,2], [3,4,5], [0,1,4,3], [1,2,5,4], [2,0,3,5]]],
            ["26", [[0,6,1,7,2,8], [3,9,4,10,5,11], [0,6,1,13,4,9,3,12],
                    [1,7,2,14,5,10,4,13], [2,8,0,12,3,11,5,14]]],
            ["5",  [[0,1,2]]],
            ["22", [[0,1,2,3,4,5]]],
            ["9",  [[0,1,2,3]]],
            ["3",  []]]
    return data

def getNodeNumber(cellType):
    num = 0
    types = cellTypesCont()
    for name in types:
        if name[0] == cellType:
            num = name[1]
            break
    return num

def getCellTypeName(cellType):
    num = ""
    types = cellTypesCont()
    for name in types:
        if name[0] == cellType:
            num = name[2]
            break
    return num

def getNodeOrder(cellType):
    order = []
    nodeOrders = nodeOrder()
    for nodeOder in nodeOrders:
        if nodeOder[0] == cellType:
            order = nodeOder[1]
            break
    return order

#
#  getNextHeaderName
#    header名（!SOLVER等）を取得
def getNextHeaderName(lines, i, headerName, itemName):
    while i < len(lines):
        if lines[i][:len(headerName)] == headerName:
            if deleteSp(lines[i]).split(",")[0] == headerName:
                if itemName == "":
                    break
                else:
                    words = itemName.split("=")
                    if abaqus2fistr.getValName(lines[i], words[0]) == words[1]:
                        break
        i += 1
    return i


#
#  getNodesData
#    nodeのデータを取得
def getNodesData(lines):
    i = getNextHeaderName(lines, 0, "!NODE", "")
    i += 1
    nodes = []
    while i < len(lines):
        if lines[i][0] == "!":
            break
        else:
            if lines[i][0] != "#":
                words = lines[i].split(",")
                line = " ".join(words[1:])
                nodes.append(line)
        i += 1
    return nodes

#
#  getNodesAllArrayData
#    nodeのdataを整理して取得
def getNodesAllArrayData(lines):
    i = 0
    nodes = []
    maxNo = 0
    while i < len(lines):
        i = getNextHeaderName(lines, i, "!NODE", "")
        i += 1
        if i < len(lines):
            while i < len(lines):
                if lines[i][0] != "#" and lines[i][:1] != "!!":
                    if lines[i][0] != "!":
                        nodes.append(lines[i])
                        nodeNo = int(lines[i].split(",")[0])
                        if maxNo < nodeNo:
                            maxNo = nodeNo
                    else:
                        break
                i += 1
    #配列を準備
    nodeConts = []
    i = 0
    while i < maxNo+1:
        nodeConts.append([])
        i += 1
    #nodeの読み込み
    for line in nodes:
        words = line.split(",")
        node = int(words[0])
        loc = " ".join(words[1:])
        nodeConts[node].append(loc)         #座標
        nodeConts[node].append(words[0])    #nodeNo
        nodeConts[node].append(0)           #使用有無
        nodeConts[node].append(-1)          #新NodeNo
    return nodeConts

#
#  getCellsData
#    cellのデータを取得
def getCellsData(lines, i, elGrpId):

    def reOrder(nodes, order):
        newNodes = []
        for num in order:
            newNodes.append(nodes[num])
        return newNodes

    i = getNextHeaderName(lines, i, "!ELEMENT", "")
    if i >= len(lines):
        return [[], i]

    cellType = getValName(lines[i], "TYPE")
    nodeOrder = getNodeOrder(cellType)
    nNode = getNodeNumber(cellType)
    cellName = getCellTypeName(cellType)
    i += 1
    cells = []
    while i < len(lines):
        if lines[i][0] == "!":
            break
        else:
            if lines[i][0] != "#":
                words = lines[i].split(",")
                newWords = []
                for word in words[1:]:
                    if word != "":
                        newWords.append(str(int(word)-1))
                newWords = reOrder(newWords, nodeOrder)
                newWords = [str(nNode)] + newWords
                cells.append([newWords, cellName, words[0], elGrpId])
        i += 1
    return [cells, i]

#
#  createVtkHeader
#
def createVtkHeader():
    lines  = ["# vtk DataFile Version 2.0" + ls]
    lines += ["Create by fistr2vtk.py" + ls]
    lines += ["ASCII" + ls]
    lines += ["DATASET UNSTRUCTURED_GRID" + ls]
    return lines

#
#  addVtkNodesData
#    vtkファイルにnodeデータを追加
def addVtkNodesData(lines, nodeConts, nNodes):
    lines += ["POINTS " + str(nNodes) + " float" + ls]
    for nodeCont in nodeConts:
        if len(nodeCont) > 2:
            if nodeCont[2] == 1:
                line = nodeCont[0]
                lines += [line]
    return lines

#
#  addVtkCellsData
#    vtkファイルにcellデータを追加
def addVtkCellsData(lines, cellConts):
    cellsNum = len(cellConts)
    allNum = 0
    newLines = []
    for cont in cellConts:
        cell = cont[0]
        cellName = cont[1]
        newLines += [" ".join(cell) + ls]
        allNum += len(cell)
    lines += ["CELLS " + str(cellsNum) + " " + str(allNum) + ls]
    lines += newLines
    return lines

#
#  addVtkCellTypes
#    vtkファイルにcellTypeを追加
def addVtkCellTypes(lines, cellConts):
    lines += ["CELL_TYPES " + str(len(cellConts)) + ls]
    for cont in cellConts:
        cellName = cont[1]
        lines += [cellName + ls]
    return lines

#  checkNanData
#    NANを0.0に修正する
def checkNanData(vals):
    newVal = []
    for val in vals:
        if val.find("NAN") >= 0:
            newVal.append("0.0")
        else:
            newVal.append(val)
    return newVal

#
#  getNodalResults
#    nodeのdataを取得
def getNodalResults(lines, i, nNode, nodeLines):
    #各変数のcomponent数を取得
    comps = lines[i].split()
    nVal = len(comps)
    nFloat = 0
    for comp in comps:
        nFloat += int(comp)
    #各変数名を取得
    i += 1
    ip = i
    valNames = []
    while i < ip+nVal:
        line = lines[i]
        valNames.append(line.split()[0])
        i += 1
    ip = i
    #headerを作成
    retLines  = ["POINT_DATA " + str(nNode) + ls]
    retLines += ["FIELD attributes " + str(nVal+1) + ls]
    #各変数分の配列を準備
    valLines = []
    j = 0
    for valName in valNames:
        line = [valName + " " + comps[j] + " " + str(nNode) + " float" + ls]
        valLines.append(line)
        j += 1
    #nodeGrp, faceGrpの配列を最後に追加
    valLines.append([])
    valLines.append([])
    #各変数の値を取得
    m = 0
    while i < len(lines) and m < nNode:
        [nums, i] = get1data(lines, i, nFloat)
        j = 0
        n = 0
        for comp in comps:
            vals = nums[n:n+int(comp)]
            #--------NANをチェック-------
            vals = checkNanData(vals)
            #---------------------------
            line = [" ".join(vals) + ls]
            valLines[j] += line
            j += 1
            n += int(comp)
        m += 1
    #nodeGroup、faceGroupのlineを追加
    valLines[0] += nodeLines[0]
    #valLines[1] += nodeLines[1]
    #各変数の値を結合
    for valLine in valLines:
        retLines += valLine
    return [retLines, i]

#
#  getElementalResults
#    elementのdataを取得
def getElementalResults(lines, i, nCell, elLines):
    #各変数のcomponent数を取得
    comps = lines[i].split()
    nVal = len(comps)
    nFloat = 0
    for comp in comps:
        nFloat += int(comp)
    #各変数名を取得
    i += 1
    ip = i
    valNames = []
    while i < ip+nVal:
        line = lines[i]
        valNames.append(lines[i].split()[0])
        i += 1
    #headerを作成
    retLines  = ["CELL_DATA " + str(nCell) + ls]
    retLines += ["FIELD attributes " + str(nVal+1) + ls]
    #各変数分の配列を準備
    ip = i
    valLines = []
    j = 0
    for valName in valNames:
        line = [valName + " " + comps[j] + " " + str(nCell) + " float" + ls]
        valLines.append(line)
        j += 1
    #elGroupを最後に追加
    valLines.append([])
    #各変数の値を取得
    m = 0
    while i < len(lines) and m < nCell:
        [nums, i] = get1data(lines, i, nFloat)
        j = 0
        n = 0
        for comp in comps:
            vals = nums[n:n+int(comp)]
            line = [" ".join(vals) + ls]
            valLines[j] += line
            j += 1
            n += int(comp)
        m += 1
    #elGroupを追加
    valLines[-1] = elLines[0]
    #各変数の値を結合
    for valLine in valLines:
        retLines += valLine
    return [retLines, i]


#  getOnlyDataInLines
#    指定行からdataのみ取得
def getOnlyDataInLines(lines, i):
    data = []
    while i < len(lines):
        if lines[i][0] != "!":
            if lines[i][0] != "#":
                words = deleteSp(lines[i]).split(",")
                for word in words:
                    if word != "":
                        data += [word]
        else:
            break
        i += 1
    return [data, i]

#  get1data
#    1行分のデータを取得
def get1data(lines, i, nFloat):
    nums = []
    while len(nums) < nFloat+1 and i < len(lines):
        nums += lines[i].split()
        i += 1
    return [nums[1:], i]

#  getValName
#    valName(TYPEなど）を取得
def getValName(line, valName):
    words = line.split(",")
    name = ""
    for word in words:
        # 「TYPE=」などを検索
        if word.find(valName+"=") >= 0:
            name = word.split("=")[1]
            break
    newName = ""
    for chara in name:
        if chara > " ":
            newName += chara
    return newName

#  deleteSp
#    空白とCRを削除して返す
def deleteSp(line):
    newLine = ""
    for chara in line:
        if chara > " ":
            newLine += chara
    return newLine

#
#  remakeLines
#    761, 781要素のdummy節点削除
def remakeLines(lines):
    for i in range(len(lines)):
        line = lines[i]
        if line[0] == "!":
            if line[:len("!ELEMENT")] == "!ELEMENT":
                elType = getValName(line, "TYPE")
                if elType == "761":
                    flag = 1
                elif elType == "781":
                    flag = 2
            else:
                flag = 0
        else:
            if flag == 1:
                words = deleteSp(line).split(",")
                newLine = ", ".join(words[:4]) + "\n"
                lines[i] = newLine
            elif flag == 2:
                words = deleteSp(line).split(",")
                newLine = ", ".join(words[:5]) + "\n"
                lines[i] = newLine
    return lines

#
#  createVtkFileMesh
#    vtkファイルのmeshデータを作成
def createVtkFileMesh(lines):
    global nodeConts, cellConts
    nodeConts = getNodesAllArrayData(lines)
    cellConts = []
    elGrpId = 1
    i = 0
    while i < len(lines):
        [cells, i] = getCellsData(lines, i, elGrpId)
        cellConts += cells
        elGrpId += 1
    nNodes = deleteExtraNodes(nodeConts, cellConts)
    nodeConts = makeNewNodeNoInConts(nodeConts)
    cellConts = renumberCellNodeArray(nodeConts, cellConts)
    newLines = createVtkHeader()
    newLines = addVtkNodesData(newLines, nodeConts, nNodes)
    newLines = addVtkCellsData(newLines, cellConts)
    newLines = addVtkCellTypes(newLines, cellConts)
    return [newLines, nNodes, len(cellConts)]

#
#  renumberCellNodeArray
#
def renumberCellNodeArray(nodeConts, cellConts):
    i = 0
    for cellCont in cellConts:
        nodes = cellCont[0]
        newNodes = []
        newNodes.append(nodes[0])
        for node in nodes[1:]:
            newNode = nodeConts[int(node)+1][3]
            newNodes.append(str(newNode))
        cellConts[i][0] = newNodes
        i += 1
    return cellConts

#
#  makeNewNodeNoInConts
#    nodeNoを順番に振り直す
def makeNewNodeNoInConts(nodeConts):
    i = 0
    No = 0
    while i < len(nodeConts):
        if len(nodeConts[i]) > 2:
            if nodeConts[i][2] == 1:
                nodeConts[i][3] = No
                No += 1
        i += 1
    return nodeConts

#
#  deleteExtraNodes
#    余分なnodeを削除する（cellで使用していないnodeを削除）
def deleteExtraNodes(nodeConts, cellConts):
    for cell in cellConts:
        nodes = cell[0][1:]
        for node in nodes:
            nodeNo = int(node)+1
            nodeConts[nodeNo][2] = 1        #使用nodeのマーク

    #使用node数を確認
    nNodes = 0
    for nodeCont in nodeConts:
        if len(nodeCont) > 2:
            if nodeCont[2] == 1:
                nNodes += 1
    return nNodes

#
#  createNodeFaceGrpVtkFile
#    nodeGrp, faceGrpのlinesを作成
def createNodeFaceGrpVtkFile(lines, nNodes):
    nodeGrpNo = getNodeGrpNo(lines, nNodes)
    [elNo, elGrpNo, elementNo] = getElementNo(lines)
    #faceGrpNo = getFaceNo(lines, nNodes, elementNo)
    #nodeLinesを取得
    nodeLines = []
    newLines = ["nodeGroup 1 ", str(nNodes) + " int" + ls]
    for node in nodeGrpNo:
        newLines += [str(node) + ls]
    nodeLines.append(newLines)
    #elementLinesを取得
    elLines = []
    nEl = str(len(elNo))
    newLines = ["elementGroup 1 " + nEl + " int" + ls]
    for el in elGrpNo:
        newLines += [str(el) + ls]
    elLines.append(newLines)
    return [nodeLines, elLines]

#
#  checkDisplacement
#    変位の他に回転が含まれている場合、回転を削除する
def checkDisplacement(lines):
    flag = 0
    for i in range(len(lines)):
        line = lines[i]
        if line[:len("DISPLACEMENT")] == "DISPLACEMENT":
            words = line.split()
            if words[1] == "3":
                flag = 1
                break
            else:
                flag = 2
                n = int(words[1])
                nData = int(words[2])
                si = i
                break
    if flag != 2:
        return lines

    header = " ".join([words[0], "3", words[2], words[3]]) + "\n"
    i += 1
    while i < len(lines):
        line = lines[i]
        if ("0" <= line[0] and line[0] <= "9") or line[0] == "." or line[0] == "-" or line[0] == "+":
            i += 1
        else:
            break
    cont = "".join(lines[si+1:i])
    nums = cont.split()
    frontLines = lines[:si]
    afterLines = lines[i:]
    addLines = [header]
    i = 0
    while i < len(nums):
        line = " ".join(nums[i:i+3]) + "\n"
        addLines.append(line)
        i += n
    newLines = frontLines + addLines + afterLines
    return newLines

#
#  createVtkFileResult
#    vtkファイルの結果データを作成
def createVtkFileResult(fileName, nNodes, nCells, grpLines):
    nodeLines = grpLines[0]
    elLines = grpLines[1]
    ans = []
    f=open(fileName); lines=f.readlines(); f.close()
    [resLines, i] = getNodalResults(lines, 3, nNodes, grpLines[0])
    ans += resLines
    if i < len(lines):
        [resLines, i] = getElementalResults(lines, i, nCells, grpLines[1])
        ans += resLines
    #DISPLACEMENTをチェック修正
    ans = checkDisplacement(ans)
    return ans

#
#  getNodeFaceCellNo
#    nodeNo, faceNo, cellNoを取得
def getNodeFaceCellNo(lines, nNodes):
    global nodeGrpNo, faceGrpNo
    nodeNo = getNodeNo(lines)
    nodeGrpNo = getNodeGrpNo(lines, nNodes)
    [elNo, elGrpNo, elementNo] = getElementNo(lines)
    #faceGrpNo = getFaceNo(lines, nNodes, elementNo)
    nodeLines = createNodeFields(nodeNo, nodeGrpNo)
    elLines = createElFields(elNo, elGrpNo)
    return nodeLines + elLines

#
#  createElFields
#    elementのfieldを作成
def createElFields(elNo, elGrpNo):
    nEl = str(len(elNo))
    newLines =  ["CELL_DATA " + nEl + ls]
    newLines += ["FIELD attribites 2" + ls]
    newLines += ["elementNo 1 " + nEl + " int" + ls]
    for el in elNo:
        newLines += [str(el) + ls]
    newLines += ["elementGroup 1 " + nEl + " int" + ls]
    for el in elGrpNo:
        newLines += [str(el) + ls]
    return newLines

#
#  createNodeFields
#    nodeのfieldを作成
def createNodeFields(nodeNo, nodeGrpNo):
    nNodes = str(len(nodeNo))
    newLines  = ["POINT_DATA " + nNodes + ls]
    newLines += ["FIELD attributes 2" + ls]
    newLines += ["nodeNo 1 " + nNodes + " int" + ls]
    for node in nodeNo:
        newLines += [str(node) + ls]
    newLines += ["nodeGroup 1 " + nNodes + " int" + ls]
    for node in nodeGrpNo:
        newLines += [str(node) + ls]
    return newLines

#  getNodeNo
#    nodeNoを取得
def getNodeNo(lines):
    #配列を準備
    nodeNo = []
    i = 0
    while i < len(nodeConts):
        if len(nodeConts[i]) > 2:
            #使用nodeかどうかcheck
            if nodeConts[i][2] == 1:
                node = nodeConts[i][1]
                nodeNo.append(node)
        i += 1
    return nodeNo

#  getNodeGrpNo
#    nodeGrpの値を取得
def getNodeGrpNo(lines, nNodes):
    #配列を準備
    nodeNo = []
    i = 0
    while i < nNodes:
        nodeNo.append(0)
        i += 1
    i = 0
    grpNo = 1
    while i < len(lines):
        i = getNextHeaderName(lines, i, "!NGROUP", "")
        [grps, i] = getOnlyDataInLines(lines, i+1)
        for node in grps:
            newNodeNo = nodeConts[int(node)][3]
            nodeNo[newNodeNo] = grpNo
        grpNo += 1
    return nodeNo

#  getElementNo
#    elementNoを取得
def getElementNo(lines):
    global cellConts
    elmNo = []
    for cont in cellConts:
        elmNo.append(int(cont[2]))
    maxElmNo = max(elmNo)
    elmNo = []
    #配列を準備
    elementNo = []
    i = 0
    while i < maxElmNo + 1:
        elementNo.append([])
        i += 1
    i = 0
    for cont in cellConts:
        elNo = int(cont[2])
        elType = cont[1]
        nodes = cont[0][1:]
        elementNo[elNo].append(i)
        elementNo[elNo].append(elType)
        elementNo[elNo].append(nodes)
        i += 1
    #elementNoのデータを取得
    elNo = []
    elGrpNo = []
    for cont in cellConts:
        elNo.append(cont[2])
        elGrpNo.append(cont[3])
    return [elNo, elGrpNo, elementNo]
    

#  getVtk2fistrElType
#    vtkのcellTypeをfistrのtypeに変換
def getVtk2fistrElType(cellType):
    typeName = ""
    types = cellTypesCont()
    for names in types:
        if names[2] == cellType:
            typeName = names[0]
            break
    return typeName

#  getFaceNo
#    faceNoを取得
def getFaceNo(lines, nNodes, elementNo):
    #配列を準備
    faceNodeNo = []
    i = 0
    while i < nNodes:
        faceNodeNo.append(0)
        i += 1
    #faceGroupを取得
    surGrp = []
    i = 0
    while i < len(lines):
        i = getNextHeaderName(lines, i, "!SGROUP", "")
        if i < len(lines):
            i += 1
            [surs, i] = getOnlyDataInLines(lines, i)
            surGrp.append(surs)
    #nodeNoを取得
    j = 0
    while j < len(surGrp):
        k = 0
        while k < len(surGrp[j]):
            element = int(surGrp[j][k])
            faceNo = int(surGrp[j][k+1])
            nodes = getNodesFromElementSurface(elementNo, element, faceNo)
            for node in nodes:
                #faceNodeNo[int(node)] = j + 1
                faceNodeNo[int(node)] = j
            k += 2
        j += 1
    return faceNodeNo

#
#  getNodesFromElementSurface
#    elementとfaceNoからnodesを取得
def getNodesFromElementSurface(elements, elementNo, faceNo):
    elType = elements[elementNo][1]
    nodesAll = elements[elementNo][2]
    nodesStd = getNodesOrderOfElement(elType, faceNo)
    nodes = []
    for node in nodesStd:
        #nodes.append(nodesAll[node-1])
        nodes.append(nodesAll[node])
    return nodes

#
#  getNodesOrderOfElement
#    faceNoからelementのnodeNoを取得
#    nodeNoの並びは、vtkの並び順
def getNodesOrderOfElement(elType, faceNo):
    data = faceNodeOfElement()
    nodes = []
    for typeFaceNo in data:
        if typeFaceNo[0] == elType:
            nodesT = typeFaceNo[1][faceNo-1]
            for node in nodesT:
                #nodes.append(node-1)        #nodeNoの起点が「1」の為、1を減算
                nodes.append(node)
            break
    return nodes

#
#  sortStepFileName
#
def sortStepFileName(names):
    steps = []
    for name in names:
        step = int(name.split(".")[-1])
        steps.append(step)
    steps.sort()
    nameHeader = ".".join(names[0].split(".")[:-1])
    names = []
    for step in steps:
        fileName = nameHeader + "." + str(step)
        names.append(fileName)
    return names

#
#  createFaceVtkFileMesh
#    表面要素のvtkを作成
def createFaceVtkFileMesh(lines, cellLines, grpLines):
    cellLines[1] = "surface" + ls
    #cellLines[3] = "DATASET POLYDATA\n"
    cellLines[3] = "DATASET UNSTRUCTURED_GRID" + ls
    ip = getHeaderPoint(cellLines, 0, "POINTS ")
    ic = getHeaderPoint(cellLines, ip, "CELLS ")
    ipd = getHeaderPoint(cellLines, ic, "POINT_DATA ")
    ie = getHeaderPoint(cellLines, ipd, "CELL_DATA ")
    nNode = ic - ip
    [polyData, faceGrpData, surElNo] = getPolygonsData(lines, nNode)
    cellTypes = getPolygonCellTypes(polyData)
    addLines  = ["CELL_DATA " + str(len(faceGrpData)) + ls]
    addLines += ["FIELD attributes 1" + ls]
    faceGrpData = addLines + faceGrpData
    cellLines = cellLines[:ic] + polyData + cellTypes + cellLines[ipd:ie] + faceGrpData
    return [cellLines, surElNo]

#
#  getPolygonCellTyps
#    表面要素のCELL_TYPESを取得
def getPolygonCellTypes(polyData):
    cellTypes = ["CELL_TYPES " + str(len(polyData)-1) + ls]
    for el in polyData[1:]:
        cellTypes += ["7" + ls]
    return cellTypes

#
#  getHeaderPoint
#    行頭の「CELLS 」の文字を検索
def getHeaderPoint(lines, i, name):
    while i < len(lines):
        n = lines[i].find(name)
        if n == 0:
            break
        i += 1
    return i

#
#  getPolygonsData
#    表面のpolygonDataを取得
def getPolygonsData(lines, nNode):
    [elNo, elGrpNo, elementNo] = getElementNo(lines)
    [faceNodes, faceGrpNo, surElNo] = getFaceNodeNo(lines, nNode, elementNo)
    newLines = []
    n = 0
    for faceNode in faceNodes:
        n += len(faceNode)
        newLines += [" ".join(faceNode) + ls]
    newLines = ["CELLS " + str(len(faceNodes)) + " " + str(n) + ls] + newLines
    grpLines = ["faceGroup 1 " + str(len(faceGrpNo)) + " int" + ls]
    for n in faceGrpNo:
        grpLines += [str(n) + ls] 
    return [newLines, grpLines, surElNo]

#
#  getFaceNodeNo
#    faceのnodeNoを取得
def getFaceNodeNo(lines, nNode, elementNo):
    #faceGroupを取得
    surGrp = []
    i = 0
    while i < len(lines):
        i = getNextHeaderName(lines, i, "!SGROUP", "")
        if i < len(lines):
            i += 1
            [surs, i] = getOnlyDataInLines(lines, i)
            surGrp.append(surs)
    #nodeNoを取得
    faceNodeNo = []
    faceGrpNo = []
    surElNo = []
    j = 0
    while j < len(surGrp):
        k = 0
        while k < len(surGrp[j]):
            element = int(surGrp[j][k])
            faceNo = int(surGrp[j][k+1])
            nodes = getNodesFromElementSurface(elementNo, element, faceNo)
            faceNodeNo.append([str(len(nodes))] + nodes)
            faceGrpNo.append(j+1)
            surElNo.append(elementNo[element][0])
            #for node in nodes:
            #    faceNodeNo[int(node)] = j
            k += 2
        j += 1
    return [faceNodeNo, faceGrpNo, surElNo]

#
#  addFaceLinesInCellLines
#    cellにfaceのlineを追加
def addFaceLinesInCellLines(cellLines, faceLines, surElNo):
    newLines = []
    ic = getHeaderPoint(cellLines, 0, "CELLS ")
    icf = getHeaderPoint(faceLines, 0, "CELLS ")
    #CELLSを追加
    words = cellLines[ic].split()
    nCells = int(words[1])
    nCellWords = int(words[2])
    words = faceLines[icf].split()
    nFaces = int(words[1])
    nFaceWords = int(words[2])
    newLine = "CELLS " + str(nCells+nFaces) + " " + str(nCellWords+nFaceWords) + ls
    cellLines[ic] = newLine
    it = getHeaderPoint(cellLines, ic, "CELL_TYPES ")
    itf = getHeaderPoint(faceLines, icf, "CELL_TYPES ")
    newLines += cellLines[:it] + faceLines[icf+1:itf]
    #CELL_TYPESを追加
    words = cellLines[it].split()
    nCells = int(words[1])
    words = faceLines[itf].split()
    nFaces = int(words[1])
    newLine = "CELL_TYPES " + str(nCells+nFaces) + ls
    cellLines[it] = newLine
    ip = getHeaderPoint(cellLines, it, "POINT_DATA ")
    ipf = getHeaderPoint(faceLines, itf, "POINT_DATA ")
    newLines += cellLines[it:ip] + faceLines[itf+1:ipf]
    #pointData部を取得
    i = getHeaderPoint(cellLines, ip, "CELL_DATA ")
    if i >= len(cellLines):
        i = len(cellLines)
        newLines += cellLines[ip:i]
    else:
        newLines += cellLines[ip:i]
        newLines += ["CELL_DATA " + str(nCells+nFaces) + ls]
        i += 1
        words = cellLines[i].split()
        newLines += [" ".join(words[:-1]) + " " + str(int(words[-1])+1) + ls]
        i += 1
        #cellDataを修正
        while i < len(cellLines):
            words = cellLines[i].split()
            dl = int(words[2])
            newLines += [words[0] + " " + words[1] + " " + str(nCells+nFaces) + " " +  words[3] + ls]
            i += 1
            newLines += cellLines[i:i+dl]
            #newData作成
            addLines = []
            if words[0] == "elementGroup":
                for j in surElNo:
                    addLines += ["0" + ls]
            else:
                for j in surElNo:
                    addLines += [cellLines[i + int(j)]]
            #dataを追加
            newLines += addLines
            i += dl
        #faceGroupを追加
        newLines += ["faceGroup 1 " + str(nCells+nFaces) + " int" + ls]
        i = 0
        while i < nCells:
            newLines += ["0" + ls]
            i += 1
        i = getHeaderPoint(faceLines, ipf, "CELL_DATA ")
        ip = getHeaderPoint(faceLines, i, "faceGroup ")
        newLines += faceLines[ip+1:ip+1+nFaces]
    return newLines

#
#  printMsg
#
def printMsg():
    msg = """

<fistr2vtk.py>------------------------------------------------------
FrontISTRフォーマットをvtkフォーマットに変換する。
FrontISTR側のmshファイルとresファイルを読み込みファイル変換する。
「-res」を省略した場合は、meshファイルのみvtk変換する。

<使い方>
fistr2vtk.py -mesh <meshFileのheader名> [-res <resFileのheader名>]

<option>
 -mesh fileHeader   meshファイル名を設定
 -res fileHeader    結果ファイル名を設定
 -h                 ヘルプ
 -help              ヘルプ

例:
fistr2vtk.py -mesh FistrModel -res FistrModel_eigen
"""
    print msg

#
#  getOption
#    optionを取得
def getOption():
    comm = ""
    meshFile = ""
    resFile = ""
    i = 0
    while i < len(sys.argv):
        if sys.argv[i] == "-mesh":
            i += 1
            meshFile = sys.argv[i]
        elif sys.argv[i] == "-res":
            i += 1
            resFile = sys.argv[i]
        i += 1

    return [meshFile, resFile]


if __name__=='__main__':

    [meshFile, resFile] = getOption()
    if meshFile == "":
        print u"error:meshファイルが入力されていません。"
        printMsg()
        exit()

    #処理開始
    meshFileName = meshFile + ".msh"
    print u"meshファイルを変換中..."
    #f=open(meshFileName); lines=f.readlines(); f.close()
    f=open(meshFileName)
    lines = []
    for line in f.readlines():
        if line != "\n":
            lines.append(line)
    f.close()
    [newLines, nNodes, nCells] = createVtkFileMesh(lines)
    if resFile == "":
        #volume要素の完成形
        newLines += getNodeFaceCellNo(lines, nNodes)
        #表面要素に関連する行を取得
        grpLines = createNodeFaceGrpVtkFile(lines, nNodes)
        [faceLines, surElNo] = createFaceVtkFileMesh(lines, newLines, grpLines)
        #newLinesに表面要素に関連する行を追加する
        addFaceLines = addFaceLinesInCellLines(newLines, faceLines, surElNo)
        #書き込み
        writeName = "conv" + meshFileName + ".vtk"
        f = open(writeName, "w")
        #cont = "".join(addFaceLines)
        #f.write(cont)
        for line in addFaceLines:
            f.write(line)
        f.close()
        print u"変換しました"
    else:
        fileName = resFile
        if len(glob.glob(fileName + ".res.1.*")):
            #並列処理の場合
            names = fileName + ".res.merge.*"
        else:
            #シングルコアの場合
            names = fileName + ".res.0.*"
        resFileNames = glob.glob(names)
        names = []
        for name in resFileNames:
            if name[-1] != "~" and name.split(".")[-1] != "inp":
                names.append(name)
        if len(names) > 0:
            names = sortStepFileName(names)
            grpLines = createNodeFaceGrpVtkFile(lines, nNodes)
            for fileName in names:
                print fileName + u" を変換中..."
                newResLines = createVtkFileResult(fileName, nNodes, nCells, grpLines)
                #volume要素の完成形
                writeLines = newLines + newResLines
                #表面要素に関連する行を取得
                [faceLines, surElNo] = createFaceVtkFileMesh(lines, writeLines, grpLines)
                #nodeConts = []; cellConts = []; nodeGrpNo = []; faceGrpNo = []
                #writeLinesに表面要素に関連する行を追加
                writeLines = addFaceLinesInCellLines(writeLines, faceLines, surElNo)
                #書き込み
                writeName = "conv" + fileName + ".vtk"
                f=open(writeName, "w")
                cont = "".join(writeLines)
                f.write(cont)
                f.close()
            print u"変換しました"
        else:
            print u"結果fileがありません"

