#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   remakeSolidShellModel.py
#
#       solidとshellモデルかどうかをチェックして、この場合は、
#       「!EQUATION」を追加する
#
#   16/06/15    新規作成
#      06/22    separateConnectFaceNodes:バグ修正
#


import sys

#                [TYPE node数]
solidElements = [["341", 4], ["342", 10],   #四面体1次2次
                ["361", 8], ["362", 20],    #六面体1次2次
                ["351", 6], ["352", 15]]    #五面体1次2次

shellElements = [["731", 3], ["761", 6],    #三角形
                ["741", 4], ["781", 8]]     #四角形

beamElements = [["611", 2], ["641", 4]]     #梁要素


nodeLoc = []        #節点Noをインデックスとして座標を取得
connectShells = []  #接続部のshell

#
#  helpMsg
#
def helpMsg():
    msg = """
<remakeSolidShellModel.py>
shellをsolidに接続する為に、shellのdummy節点（回転角）とsolid側shellの接続面
の変位の関係式を「!EQUATION」で記述し、mshファイルに追記する。
追記する場所は、!NODE定義の最後（!ELEMENT定義の前）。
この場所に既に!EQUATIONが存在する場合は、これを置き換える。
これにより、solidの回転角と変位がshell側に伝わっていく。
shellのみ、solidのみのmshファイルの場合は、なにもせず戻る。
mshファイル内に入力したconnectFace名が存在しない場合も、なにもせず戻る。

<使い方>
remakeSolidShellModel.py <mshFile> [<connectFace>]

mshFile      :mshファイル
connectFace  :solid側のshellとの接続面の節点グループ名
              省略時は、"shellConnect"が設定される

例：
remakeSolidShellModel.py FistrModel.msh
remakeSolidShellModel.py FistrModel.msh shellConnect
"""
    print msg


#
#  crossProduct
#    ベクトル積（外積）
def crossProduct(a, b):
    x = a[1]*b[2] - a[2]*b[1]
    y = a[2]*b[0] - a[0]*b[2]
    z = a[0]*b[1] - a[1]*b[0]
    return [x, y, z]

#
#  dotProduct
#    スカラー積（内積）
def dotProduct(a, b):
    ans = a[0]*b[0] + a[1]*b[1] + a[2]*b[2]
    return ans

#
#  mag
#    vectorの絶対値の取得
def mag(a):
    ans = pow((a[0]*a[0] + a[1]*a[1] + a[2]*a[2]), 0.5)
    return ans

#  getLength
#    2点間の距離を算出
def getLength(a, b):
    vec = getVector(a, b)
    l = mag(vec)
    return l

#  getVector
#    2点間のベクトルを作成
def getVector(a, b):
    x = b[0] - a[0]
    y = b[1] - a[1]
    z = b[2] - a[2]
    return [x,y,z]

#  normalVector
#    単位ベクトルを取得
def normalVector(vec):
    l = mag(vec)
    x = vec[0]/l
    y = vec[1]/l
    z = vec[2]/l
    return [x,y,z]


#  readLines
#    fileの読み込み
def readLines(fileName):
    f = open(fileName)
    lines = f.readlines()
    f.close()
    return lines

#  isSolidElement
#    solid要素?
def isSolidElement(elName):
    ans = False
    for element in solidElements:
        if elName == element[0]:
            ans = True
            break
    return ans

#  isShellElement
#    shell要素?
def isShellElement(elName):
    ans = False
    for element in shellElements:
        if elName == element[0]:
            ans = True
            break
    return ans

#  isSolidShellModel
#    solidShellかどうかチェック
def isSolidShellModel(lines):
    solid = False
    shell = False
    for line in lines:
        if line[:len("!ELEMENT")] == "!ELEMENT":
            typeName = getValName(line, "TYPE")
            if isSolidElement(typeName) == True:
                solid = True
            elif isShellElement(typeName) == True:
                shell = True
        if solid == True and shell == True:
            break
    if solid == True and shell == True:
        return True
    else:
        return False

#  isShellConnectNodes
#    shellConnectのnodeGroupがあるか?
def isShellConnectNodes(lines, connectFace):
    ans = False
    for line in lines:
        if line[:len("!NGROUP")] == "!NGROUP":
            ngrp = getValName(line, "NGRP")
            if ngrp == connectFace:
                ans = True
                break
    return ans

#
#  getValName
#    lineからvalNameを取得
def getValName(line, valName):
    words = line.split(",")
    name = ""
    for word in words:
        # 「TYPE=」などを検索
        if word.find(valName+"=") >= 0:
            name = word.split("=")[1]
            break
    newName = ""
    for chara in name:
        if chara > " ":
            newName += chara
    return newName

#
#  deleteSp
#    空白とCRを削除して返す
def deleteSp(line):
    newLine = ""
    i = 0
    for chara in line:
        if chara > " ":
            newLine += chara
        else:
            if i >= 1 and chara == " ":
                if line[i-1] == " " or line[i-1] == ",":
                    pass
                else:
                    newLine += chara
        i += 1 
    return newLine

#
#  getNodeLocation
#    nodeの座標を取得
def getNodeLocation(lines):
    #nodeの行を取得
    nodeLines = []
    flag = 0
    for line in lines:
        if line[0] == "!":
            if line[:len("!NODE")] == "!NODE":
                flag = 1
            else:
                flag = 0
        else:
            if flag == 1:
                words = deleteSp(line).split(",")
                nodeNo = int(words[0])
                x = float(words[1])
                y = float(words[2])
                z = float(words[3])
                nodeLines.append([nodeNo,x,y,z])
    nodeLines.sort()
    maxNodeNo = nodeLines[-1][0]
    nodeLoc = []
    for i in range(len(nodeLines)+1):
        nodeLoc.append([])
    for node in nodeLines:
        nodeNo = node[0]
        nodeLoc[nodeNo] = node[1:]
    return nodeLoc            

#
#  getShellConnectNodes
#    shellConnect面のnodeを取得
def getShellConnectNodes(lines, grpName):
    nodeLines = []
    flag = 0
    for line in lines:
        if line[0] == "!":
            if line[:len("!NGROUP")] == "!NGROUP":
                name = getValName(line, "NGRP")
                if name == grpName:
                    flag = 1
                else:
                    flag = 0
            else:
                flag = 0
        else:
            if flag == 1:
                words = deleteSp(line).split(",")
                for word in words:
                    try:
                        a = int(word)
                        nodeLines += [a]
                    except:
                        pass
    return nodeLines

#  delSameNodes
#    同じnodeNoを削除
def delSameNodes(nodes):
    nodes.sort()
    newNodes = [nodes[0]]
    for node in nodes:
        if newNodes[-1] != node:
            newNodes.append(node)
    return newNodes

#
#  getShellNodesOnConnect
#    shell接続面上のshellの節点を取得
def getShellNodesOnConnect(lines, shellConnect):
    #shell行を取得
    shellLines = []
    flag = 0
    for line in lines:
        if line[0] == "!":
            if line[:len("!ELEMENT")] == "!ELEMENT":
                typeName = getValName(line, "TYPE")
                if isShellElement(typeName) == True:
                    flag = 1
            else:
                flag = 0
        else:
            if flag == 1:
                shellLines.append(line)
    #数字に変換
    shellEl = []
    for line in shellLines:
        words = deleteSp(line).split(",")
        data = []
        for word in words:
            try:
                a = int(word)
                data += [a]
            except:
                pass
        shellEl.append(data)
    #connect面のnodeを取得
    #  conNodes：[[shellNode, dummyNode], ..]
    conNodes = []; shells = []
    for shell in shellEl:
        flag = 0
        for i in range(len(shell[1:])):
            shellNode = shell[i+1]
            for cnNode in shellConnect:
                if shellNode == cnNode:
                    p = len(shell[1:])/2 + i + 1
                    conNodes.append([shellNode, shell[p]])
                    flag = 1
                    break
        if flag == 1:
            shells.append(shell)
    conNodes = delSameNodes(conNodes)
    return [conNodes, shells]

#
# separateConnectFaceNodes
#    shellの接続面の節点を裏表に分ける
def separateConnectFaceNodes(faceNodes, nodes, shells):

    def isInShellNode(node, nVecs):
        ans = False
        for nVec in nVecs:
            for No in nVec[1]:
                if node == No:
                    ans = True
                    break
        return ans

    def deleteSameNode(nodes):
        nodes.sort()
        newNodes = [nodes[0]]
        for node in nodes:
            if newNodes[-1][0] != node[0]:
                newNodes.append(node)
        return newNodes

    upperNodes = []
    lowerNodes = []
    nVecs = []
    #shellの単位ベクトルを作成
    for i in range(len(shells)):
        shell = shells[i]
        loc1 = nodeLoc[shell[1]]
        loc2 = nodeLoc[shell[2]]
        vec1 = getVector(loc1, loc2)
        loc1 = nodeLoc[shell[2]]
        loc2 = nodeLoc[shell[3]]
        vec2 = getVector(loc1, loc2)
        vec = crossProduct(vec1, vec2)
        nVec = normalVector(vec)
        sameNodes = []
        nNodes = (len(shell) - 1) / 2
        for n in range(nNodes):
            shellNode = shell[n+1]
            for faceNode in faceNodes:
                if shellNode == faceNode:
                    sameNodes.append(shellNode)
                    break
        if len(sameNodes) == 2:
            nVecs.append([nVec, sameNodes])
    #nodeを表裏に分ける
    up = []; low = []
    for faceNode in faceNodes:
        if isInShellNode(faceNode, nVecs) == False:
            #最短のnodeNoを調べる
            nearNodes = []
            for nVec in nVecs:
                pnt1 = nVec[1][0]
                pnt2 = nVec[1][1]
                l1 = getLength(nodeLoc[pnt1], nodeLoc[faceNode])
                l2 = getLength(nodeLoc[pnt2], nodeLoc[faceNode])
                nearNodes.append([l1, pnt1, nVec])
                nearNodes.append([l2, pnt2, nVec])
            nearNodes.sort()
            nearNode = nearNodes[0][1]
            #裏と表を調べる
            vec = getVector(nodeLoc[nearNode], nodeLoc[faceNode])
            nVec = nearNodes[0][2]
            dotPro = dotProduct(vec, nVec[0])
            if dotPro >= 0:
                #表側
                upperNodes.append([faceNode, nVec])
            else:
                #裏側
                lowerNodes.append([faceNode, nVec])
    return [upperNodes, lowerNodes]

#
#  getUnsetDummyNodes
#    セットされていないdummy節点を取得する
def getUnsetDummyNodes(connectNodes, setDummyNodes):
    unsetNodes = []
    for [node, dummy] in connectNodes:
        flag = 0
        for setDummyNode in setDummyNodes:
            if dummy == setDummyNode:
                flag = 1
                break
        if flag == 0:
            unsetNodes.append([node, dummy])
    return unsetNodes

#
#  createEquationNodesLines
#    表側の節点の!EQUATIONを作成
def createEquationNodesLines(nodes, connectNodes):

    #  createLine
    #    追加する行を作成
    def createLine(direct, faceNode, shellNode, rDirect1, dummyNode1, r1,
                   rDirect2, dummyNode2, r2):
        lineList = [str(faceNode), direct, "1", str(shellNode), direct, "-1", 
                    str(dummyNode1), rDirect1, str(-r1),
                    str(dummyNode2), rDirect2, str(-r2)]
        line = ", ".join(lineList) + "\n"
        return line

    #メイン
    newLinesx = []
    newLinesy = []
    newLinesz = []
    #各方向の単位ベクトル
    ox = [1.0, 0.0, 0.0]
    oy = [0.0, 1.0, 0.0]
    oz = [0.0, 0.0, 1.0]
    setDummyNodes = []
    #faceNode毎にチェック
    for [node, [nFaceVec, pnts]] in nodes:
        #node    :faceの節点
        #nFaceVec:shellの向き（表裏）
        #pnts    :shellと共有するedge（2ヶの節点）
        #shellのedgeの単位ベクトル取得

        #最短のshellNode取得
        l1 = getLength(nodeLoc[node], nodeLoc[pnts[0]])
        l2 = getLength(nodeLoc[node], nodeLoc[pnts[1]])
        if l1 <= l2:
            shellNode = pnts[0]
        else:
            shellNode = pnts[1]
        #shellEdgeの単位ベクトル取得
        vec = getVector(nodeLoc[pnts[0]], nodeLoc[pnts[1]])
        nVec = normalVector(vec)
        #dummy節点を取得
        for cNode in connectNodes:
            if cNode[0] == shellNode:
                dummyNode = cNode[1]    #dummy節点
                setDummyNodes.append(dummyNode)
                break
        #接続点からのベクトル
        vec = getVector(nodeLoc[shellNode], nodeLoc[node])
        #接続ラインからの距離（ベクトル）
        r = dotProduct(vec, nFaceVec)
        rVec = [nFaceVec[0]*r, nFaceVec[1]*r, nFaceVec[2]*r]

        #x方向変位
        ry = dotProduct(rVec, oz)
        rz = -dotProduct(rVec, oy)
        newLinesx += ["4, 0.0\n"]
        newLinesx += [createLine("1", node, shellNode, "2",
                        dummyNode, ry, "3", dummyNode, rz)]
        #y方向変位
        rz = dotProduct(rVec, ox)
        rx = -dotProduct(rVec, oz)
        newLinesy += ["4, 0.0\n"]
        newLinesy += [createLine("2", node, shellNode, "3",
                        dummyNode, rz, "1", dummyNode, rx)]
        #z方向変位
        rx = dotProduct(rVec, oy)
        ry = -dotProduct(rVec, ox)
        newLinesz += ["4, 0.0\n"]
        newLinesz += [createLine("3", node, shellNode, "1",
                        dummyNode, rx, "2", dummyNode, ry)]
    #セットされていないdummy節点を取得
    unsetDummyNodes = getUnsetDummyNodes(connectNodes, setDummyNodes)
    if len(unsetDummyNodes) > 0:
        print u"未セットのdummy接点「" + str(len(unsetDummyNodes)) + u"」ヶにEQUATIONをセットします。"
        #未セットのdummy節点にEQUATIONをセット
        addLinesxyz = createUnsetDummyNode(nodes, connectNodes, unsetDummyNodes)
        newLinesx += addLinesxyz[0]
        newLinesy += addLinesxyz[1]
        newLinesz += addLinesxyz[2]
    newLines = newLinesx + newLinesy + newLinesz
    return newLines

#
#  createUnsetDummyNode
#    dummy接点を基準にEQUATIONを作成
def createUnsetDummyNode(nodes, connectNodes, unsetDummyNodes):

    #  createLine
    #    追加する行を作成
    def createLine(direct, faceNode, shellNode, rDirect1, dummyNode1, r1,
                   rDirect2, dummyNode2, r2):
        lineList = [str(faceNode), direct, "1", str(shellNode), direct, "-1", 
                    str(dummyNode1), rDirect1, str(-r1),
                    str(dummyNode2), rDirect2, str(-r2)]
        line = ", ".join(lineList) + "\n"
        return line

    #メイン
    newLinesx = []
    newLinesy = []
    newLinesz = []
    #各方向の単位ベクトル
    ox = [1.0, 0.0, 0.0]
    oy = [0.0, 1.0, 0.0]
    oz = [0.0, 0.0, 1.0]
    #dummyNode毎にチェック
    for [shellNode, dummyNode] in unsetDummyNodes:
        #最短のnodeを取得
        lengthes = []
        for [node, [nFaceVec, pnts]] in nodes:
            l = getLength(nodeLoc[shellNode], nodeLoc[node])
            lengthes.append([l, [node, [nFaceVec, pnts]]])
        lengthes.sort()
        [l, [node, [nFaceVec, pnts]]] = lengthes[0]
        #shellのedgeの単位ベクトル取得
        vec = getVector(nodeLoc[pnts[0]], nodeLoc[pnts[1]])
        nVec = normalVector(vec)
        #接続点からのベクトル
        vec = getVector(nodeLoc[shellNode], nodeLoc[node])
        #接続ラインからの距離（ベクトル）
        r = dotProduct(vec, nFaceVec)
        rVec = [nFaceVec[0]*r, nFaceVec[1]*r, nFaceVec[2]*r]

        #x方向変位
        ry = dotProduct(rVec, oz)
        rz = -dotProduct(rVec, oy)
        newLinesx += ["4, 0.0\n"]
        newLinesx += [createLine("1", node, shellNode, "2",
                        dummyNode, ry, "3", dummyNode, rz)]
        #y方向変位
        rz = dotProduct(rVec, ox)
        rx = -dotProduct(rVec, oz)
        newLinesy += ["4, 0.0\n"]
        newLinesy += [createLine("2", node, shellNode, "3",
                        dummyNode, rz, "1", dummyNode, rx)]
        #z方向変位
        rx = dotProduct(rVec, oy)
        ry = -dotProduct(rVec, ox)
        newLinesz += ["4, 0.0\n"]
        newLinesz += [createLine("3", node, shellNode, "1",
                        dummyNode, rx, "2", dummyNode, ry)]
    newLines = [newLinesx, newLinesy, newLinesz]
    return newLines

#
#  createEquationLines
#    「!EQUATION」を作成
def createEquationLines(lines, faceName):
    global nodeLoc
    #節点座標の読み込み
    nodeLoc = getNodeLocation(lines)
    #shellとの接続面の節点を取得
    connectFaceNodes = getShellConnectNodes(lines, faceName)
    print u"接続面の節点数: " + str(len(connectFaceNodes)) + u"  （共有節点含む）"
    #接続面に接続されるshellと節点を取得
    [connectNodes, connectShells] = getShellNodesOnConnect(lines, connectFaceNodes)
    print u"solidShell共有節点数: " + str(len(connectNodes))
    #接続面の節点をshellの表裏側に分け、その節点に近いshellとshell節点を取得
    [upperNodes, lowerNodes] = separateConnectFaceNodes(connectFaceNodes,
                                    connectNodes, connectShells)
    #追加するEQUATIONの行を作成
    upperAddLines = createEquationNodesLines(upperNodes, connectNodes)
    lowerAddLines = createEquationNodesLines(lowerNodes, connectNodes)
    addLines = ["!EQUATION\n"] + upperAddLines + lowerAddLines
    return addLines

#  skipNextHeader
#    次のheaderまでskipする
def skipNextHeader(lines, i):
    ii = len(lines)
    while i < len(lines):
        if lines[i][0] == "!":
            ii = i
            break
        i += 1
    return ii

#
#  replaceEquationLines
#  --------------------
#    !EQUATIONのblockを置き換える
#    場所は、!NODEと!ELEMENTの間（!ELEMENTの前）
def replaceEquationLines(lines, addLines):
    i = 0
    while i < len(lines):
        i = skipNextHeader(lines, i)
        if lines[i][:len("!EQUATION")] == "!EQUATION":
            ps = i
            while i < len(lines):
                i = skipNextHeader(lines, i)
                if lines[i][:len("!ELEMENT")] == "!ELEMENT":
                    pe = i
                    newLines = lines[:ps] + addLines + lines[pe:]
                    break
                i += 1
            break
        elif lines[i][:len("!ELEMENT")] == "!ELEMENT":
            ps = i
            newLines = lines[:ps] + addLines + lines[ps:]
            break
        i += 1
    return newLines

#
#  checkAndSetEquation
#    solidShell混在モデルかどうかをチェックし、混在モデルの場合はEQUATIONをセット
def checkAndSetEquation(lines, fileName, connectFace):
    if isSolidShellModel(lines) == False:
        return [False, lines]
    if isShellConnectNodes(lines, connectFace) == False:
        return [False, lines]

    print u"\nshellのdummy節点にEQUATIONを設定します。"
    addLines = createEquationLines(lines, connectFace)
    newLines = replaceEquationLines(lines, addLines)
    return [True, newLines]    

#
#  remakeSolidShellModel
#  ---------------------
#    solidShellモデルの場合、EQUATIONを追加する。
def remakeSolidShellModel(fileName, connectFace):
    lines = readLines(fileName)
    #EQUATIONを追加
    [flag, newLines] = checkAndSetEquation(lines, fileName, connectFace)
    if flag == True:
        #書き込み
        f=open(fileName, "w")
        for line in newLines:
            f.write(line)
        f.close()
        print u"solidとshellの接続面の「!EQUATION」を設定しました。"


if __name__=='__main__':
    if len(sys.argv) == 1:
        helpMsg()
        exit()
    elif len(sys.argv) == 2:
        if sys.argv[1] == "-h" or sys.argv[1] == "-help" or sys.argv[1] == "--help":
            helpMsg()
            exit()
        mshFile = sys.argv[1]
        connectFace = "shellConnect"
    else:
        mshFile = sys.argv[1]
        connectFace = sys.argv[2]
    remakeSolidShellModel(mshFile, connectFace)

