#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
#       主応力の計算
#
#   15/12/17    主応力のベクトル追加
#      12/24    精度向上（有効桁数10桁）、内容を整理
#   16/01/30    全面見直し（主値と主軸のアンマッチの可能性ある為）
#               以下のwebを参考にしてpythonに書き換え。
#               http://www.fem.gr.jp/fem/3dim/3dsolidexample/3dsolidexampleb.html
#


import math

maxIta = 100
eps = 1e-14

#
#  principalStress for Newton-Raphson
#
def principalStress(sigmaList):
    [xx, yy, zz, xy, yz, xz] = sigmaList
    A = xx + yy + zz
    B = xx*yy + yy*zz + zz*xx - xy*xy -yz*yz -xz*xz
    C = xx*yy*zz + 2.0*xy*xz*yz - xy*xy*zz - yz*yz*xx - xz*xz*yy

    energyY0 = (xx-yy)*(xx-yy) + (yy-zz)*(yy-zz) + (zz-xx)*(zz-xx) + 6.0*(xy*xy + yz*yz + xz*xz)

    sv1 = [1.0, 0.0, 0.0]
    sv2 = [0.0, 1.0, 0.0]
    sv3 = [0.0, 0.0, 1.0]
    
    if A == 0.0 and B == 0.0 and C == 0.0:
        s1 = 0.0
        s2 = 0.0
        s3 = 0.0
        return [s1, s2, s3, sv1, sv2, sv3]

    elif B == 0.0 and C == 0.0:
        s2 = 0.0
        if A > 0.0:
            s1 = A
            s3 = 0.0
        else:
            s1 = 0.0
            s3 = A
        return [s1, s2, s3, sv1, sv2, sv3]

    elif C == 0.0:
        s1 = (A + pow((A*A - 4.0*B), 0.5)) / 2.0
        s2 = 0.0
        s3 = (A - pow((A*A - 4.0*B), 0.5)) / 2.0
        if s3 > 0.0:
            s2 = s3
            s3 = 0.0
        P = -(-A)*(-A) / 3.0 + B
        Q = 2.0 * (-A)*(-A)*(-A) / 27.0 - (-A)*B/3.0 + (-C)
        if P == 0.0 and Q == 0.0:
            s1 = xx
            s2 = yy
            s3 = zz
        return [s1, s2, s3, sv1, sv2, sv3]

    s1 = pow(2.0, 0.5) + A/3.0
    s2 = pow(3.0, 0.5) + B/s1
    s3 = math.atan(1.0) + C/(s1*s2)

    energyY11 = energyY0

    for i in range(maxIta):
        ic = i
        f1 = s1 + s2 + s3 - A
        f2 = s1*s2 + s2*s3 + s3*s1 - B
        f3 = s1*s2*s3 - C

        drv11 = 1.0
        drv12 = 1.0
        drv13 = 1.0
        drv21 = s2 + s3
        drv22 = s1 + s3
        drv23 = s1 + s2
        drv31 = s2*s3
        drv32 = s1*s3
        drv33 = s1*s2

        detj = ( drv11 * (drv22*drv33 - drv23*drv32)
               - drv12 * (drv21*drv33 - drv23*drv31)
               + drv13 * (drv21*drv32 - drv22*drv31) )

        di11 = (drv22*drv33 - drv23*drv32) / detj
        di21 = (drv23*drv31 - drv21*drv33) / detj
        di31 = (drv21*drv32 - drv22*drv31) / detj
        di12 = (drv13*drv32 - drv12*drv33) / detj
        di22 = (drv11*drv33 - drv13*drv31) / detj
        di32 = (drv12*drv31 - drv11*drv32) / detj
        di13 = (drv12*drv23 - drv13*drv22) / detj
        di23 = (drv13*drv21 - drv11*drv23) / detj
        di33 = (drv11*drv22 - drv12*drv21) / detj

        s1 = s1 - (di11*f1 + di12*f2 + di13*f3)
        s2 = s2 - (di21*f1 + di22*f2 + di23*f3)
        s3 = s3 - (di31*f1 + di32*f2 + di33*f3)

        energyY1 = ( 2.0*(s1 + s2 + s3)*(s1 + s2 + s3)
                    -6.0*(s1*s2* + s2*s3 + s3*s1) )

        if abs(energyY1 - energyY11)/energyY0 < eps:
            break
        else:
            energyY11 = energyY1
    if i >= maxIta-1:
        ic = -1
    #ベクトルを計算
    [sv1, sv2, sv3] = sVector(sigmaList, [s1, s2, s3])
    [s1, s2, s3, sv1, sv2, sv3] = reorderSS(s1, s2, s3, sv1, sv2, sv3)
    return [s1, s2, s3, sv1, sv2, sv3]

#
#  sVector  各方向成分
#
def sVector(sigmaList, si):
    [xx, yy, zz, xy, yz, xz] = sigmaList

    U = []
    for i in range(3):
        v = []
        for j in range(3):
            v.append(0.0)
        U.append(v)
    
    for mode in range(3):
        f11 = xx - si[mode]*(1.0-eps)
        f22 = yy - si[mode]*(1.0-eps)
        f33 = zz - si[mode]*(1.0-eps)
        f12 = xy
        f13 = xz
        f23 = yz
        f21 = f12
        f31 = f13
        f32 = f23

        posi = f11*f22*f33 + f21*f32*f13 + f31*f12*f23
        aneg = f13*f22*f31 + f11*f23*f32 + f21*f12*f33
        det = posi - aneg

        g11 =  (f22*f33 - f23*f32)/det
        g12 = -(f21*f33 - f23*f31)/det
        g13 =  (f21*f32 - f22*f31)/det

        g21 = -(f12*f33 - f13*f32)/det
        g22 =  (f11*f33 - f13*f31)/det
        g23 = -(f11*f32 - f12*f31)/det

        g31 =  (f12*f23 - f13*f22)/det
        g32 = -(f11*f23 - f13*f21)/det
        g33 =  (f11*f22 - f12*f21)/det

        g11 = si[mode]*eps*g11
        g12 = si[mode]*eps*g12
        g13 = si[mode]*eps*g13
        g21 = si[mode]*eps*g21
        g22 = si[mode]*eps*g22
        g23 = si[mode]*eps*g23
        g31 = si[mode]*eps*g31
        g32 = si[mode]*eps*g32
        g33 = si[mode]*eps*g33

        U[mode][0] = 0.0
        U[mode][1] = 0.0
        U[mode][2] = 0.0
        U[mode][mode] = 1.0

        vecl = 9999.0
        for i in range(maxIta):
            tempU1 = g11*U[mode][0] + g12*U[mode][1] + g13*U[mode][2]
            tempU2 = g21*U[mode][0] + g22*U[mode][1] + g23*U[mode][2]
            tempU3 = g31*U[mode][0] + g32*U[mode][1] + g33*U[mode][2]

            vecLength = pow((tempU1*tempU1 + tempU2*tempU2 + tempU3*tempU3), 0.5)

            U[mode][0] = tempU1/vecLength
            U[mode][1] = tempU2/vecLength
            U[mode][2] = tempU3/vecLength

            delta = abs(vecl - vecLength) / max(vecl, vecLength)

            if delta <= eps:
                break
            vecl =vecLength

    for mode in range(3):
        l = abs(si[mode])
        for i in range(3):
            U[mode][i] = U[mode][i] * l
    return [U[0], U[1], U[2]]

#
#  ordering
#    昇順に並べ替え
#
#  reorderSS
#    最大、最小に並べ替え
def reorderSS(SS1, SS2, SS3, SSV1, SSV2, SSV3):
    if SS1 > SS2 and SS2 > SS3:
        Sigm1 = SS1; sigma1 = SSV1
        Sigm2 = SS2; sigma2 = SSV2
        Sigm3 = SS3; sigma3 = SSV3
    elif SS1 > SS2 and SS3 > SS2 and SS1 > SS3:
        Sigm1 = SS1; sigma1 = SSV1
        Sigm2 = SS3; sigma2 = SSV3
        Sigm3 = SS2; sigma3 = SSV2
    elif SS2 > SS1 and SS2 > SS3 and SS1 > SS3:
        Sigm1 = SS2; sigma1 = SSV2
        Sigm2 = SS1; sigma2 = SSV1
        Sigm3 = SS3; sigma3 = SSV3
    elif SS2 > SS1 and SS2 > SS3 and SS3 > SS1:
        Sigm1 = SS2; sigma1 = SSV2
        Sigm2 = SS3; sigma2 = SSV3
        Sigm3 = SS1; sigma3 = SSV1
    elif SS2 > SS1 and SS3 > SS2:
        Sigm1 = SS3; sigma1 = SSV3
        Sigm2 = SS2; sigma2 = SSV2
        Sigm3 = SS1; sigma3 = SSV1
    elif SS3 > SS1 and SS1 > SS2:
        Sigm1 = SS3; sigma1 = SSV3
        Sigm2 = SS1; sigma2 = SSV1
        Sigm3 = SS2; sigma3 = SSV2
    return [Sigm1, Sigm2, Sigm3, sigma1, sigma2, sigma3]
        


if __name__ == "__main__":
    #xx=4.61E+02; yy=1.50E+02; zz=2.07E+02
    #xy=-3.13E+01; yz=-1.15E+01; zx=-6.62E+01
    xx=4.61E+02; yy=1.50E+02; zz=2.07E+02
    xy=-3.13E+01; yz=1.15E+01; zx=6.62E+01

    sigmaList = [xx, yy, zz, xy, yz, zx]
    [sig1, sig2, sig3, sigV1, sigV2, sigV3] = principalStress(sigmaList)
    mises2 = (pow((sig1-sig2),2) + pow((sig2-sig3),2) + pow((sig3-sig1),2))/2.0
    mises = pow(mises2, 0.5)
    print sig1, sig2, sig3, [sigV1, sigV2, sigV3]

